     //////////////////////////////////////////////////////////////////////
     //
     // Copyright (C) 1998 Analytic Services, Inc. (ANSER)
     //
     // Permission is hereby granted, free of charge, to any person
     // obtaining a copy of this software and associated documentation
     // files (the "Software"), to deal in the Software without
     // restriction, including without limitation the rights to use, copy,
     // modify, merge, publish, distribute, sublicense, and/or sell copies
     // of the Software, and to permit persons to whom the Software is
     // furnished to do so, subject to the following conditions:
     //
     // The above copyright notice and this permission notice shall be
     // included in all copies or substantial portions of the Software.
     //
     // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
     // EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
     // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
     // NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
     // HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
     // WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
     // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
     // DEALINGS IN THE SOFTWARE.
     //
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mit.app.ajax;

     import java.io.*;
     import java.net.*;
     import java.util.*;
     import java.security.MessageDigest;
     import java.security.acl.Group;

     import javax.servlet.*;
     import javax.servlet.http.*;

     import com.sun.server.realm.*;
     import com.sun.server.realm.sharedpassword.*;

     import com.orbs.open1.mit.lang.StringLib;
     import com.orbs.open1.mit.net.mail.SMTPMail;
// Get rid of SMTPTest.
     import com.orbs.open1.mit.net.mail.SMTPTest;
     import com.orbs.open1.mit.servlet.ServletLib;

     /*********************************************************************
     *
     * Adds the user to the Java Web Server and creates and stores
     * a corresponding UserData object.
     *
     * <P>
     *
     * To use this, the Java Web Server must be prepped in the following
     * ways:<BR>
     * <UL>
     * <LI> In the "defaultRealm", create a group called "AgentUserGroup".
     * <LI> Create an ACL such as "agentUserAcl" and assign AgentUserGroup
     *      to it.
     * <LI> Give the group "get" and "post" privileges in the ACL.
     * <LI> Protect your servlet resources using agentUserAcl.
     * </UL>
     *
     * @see
     *   UserManager
     * @see
     *   UserData
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     * @version
     *   1998-10-29
     *********************************************************************/

     public class  RegistrationServlet extends AJAXServlet
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     public static final String  NAME    = "RegistrationServlet";
     public static final String  TITLE   = "Registration";
     public static final double  VERSION = 0.6;
     public static final String  INFO
       = NAME + " v" + VERSION + " Copyright 1998 ANSER";
     public static final String  ACTION_URL
       = "/servlet/registrationservlet";

     public static final String  PARAM_USER = "USER";
     public static final String  PARAM_PAS1 = "PAS1";
     public static final String  PARAM_PAS2 = "PAS2";
     public static final String  PARAM_MAIL = "MAIL";
     public static final String  PARAM_CODE = "CODE";

     public static final String  ATTR_MAIL  = "MAIL";

     public static final String  REALM_NAME = "defaultRealm";
     public static final String  GROUP_NAME = "AgentUserGroup";

     private static final String  SUBJECT   = TITLE;

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     /*********************************************************************
     * Test method.  (username, password, e-mail)
     *********************************************************************/
     public static void  main ( String [ ]  args )
     //////////////////////////////////////////////////////////////////////
     {
       System.out.println (
         validation_code ( args [ 0 ], args [ 1 ], args [ 2 ] ) );
     }

     public String  getServletInfo ( )
     //////////////////////////////////////////////////////////////////////
     {
       return INFO;
     }
  
     //////////////////////////////////////////////////////////////////////
     // Abstract method implementations
     //////////////////////////////////////////////////////////////////////

     /*********************************************************************
     * Returns the title to be used with the HTML header tag TITLE.
     *********************************************************************/
     protected String  getTitle ( ) { return TITLE; }

     /*********************************************************************
     * Returns the relative path and filename for the servlet-specific
     * header file.
     *********************************************************************/
     protected String  getHeaderAlias ( )
       { return "/java/app/registration/header.html"; }

     /*********************************************************************
     * Returns the relative path and filename for the servlet-specific
     * footer file.
     *********************************************************************/
     protected String  getFooterAlias ( )
       { return "/java/app/registration/footer.html"; }

     /*********************************************************************
     * Returns true if the servlet requires user authentication.
     *********************************************************************/
     protected boolean  requiresUserData ( ) { return false; }

     /*********************************************************************
     * Called by doPost() to provide the servlet-specific services.
     *********************************************************************/
     protected void  processRequest (
       HttpServletRequest   req,
       HttpServletResponse  res,
       UserData             userData )
       throws IOException, ServletException
     //////////////////////////////////////////////////////////////////////
     {
       PrintStream  out = new PrintStream ( res.getOutputStream ( ) );
       try
       {
       String  user = ServletLib.getParameter ( req, PARAM_USER );
       String  pass = ServletLib.getParameter ( req, PARAM_PAS1 );
       String  pas2 = ServletLib.getParameter ( req, PARAM_PAS2 );
       String  mail = ServletLib.getParameter ( req, PARAM_MAIL );
       String  code = ServletLib.getParameter ( req, PARAM_CODE );

       if ( ( user == null )
         || ( pass == null )
         || ( mail == null ) )
       {
         page_register ( out, user, pass, mail );
         return;
       }

       char [ ]  usertest = user.toCharArray ( );
       for ( int  i = 0; i < usertest.length; i++ )
       {
         if ( !Character.isLetter ( usertest [ i ] ) )
         {
           out.println (
             "<B>User name characters must be all letters.</B><P>" );
           page_register ( out, null, null, mail );
           return;
         }
       }

       if ( !pass.equals ( pas2 ) )
       {
         out.println ( "<B>Passwords do not match.</B><P>" );
         page_register ( out, user, null, mail );
         return;
       }

       SharedPasswordRealm  realm = getRealm ( );

       try
       {
         realm.getUser ( user );
         if ( getUserManager ( ).getUserData ( user ) != null )
         {
           out.println ( "<B>Username \"" + user
             + "\" already in use.</B><P>" );
           page_register ( out, "", pass, mail );
           return;
         }
       }
       catch ( NoSuchUserException  e ) { }

       if ( !pass.equals ( pas2 ) )
       {
         out.println ( "<B>Passwords do not match.</B><P>" );
         page_register ( out, user, null, mail );
         return;
       }

       if ( ( code == null ) || ( code.length ( ) == 0 ) )
       {
         String  message
           = "\r\n"
           + "Username:  " + user + "\r\n"
           + "Your validation code is:\r\n"
           + validation_code ( user, pass, mail ) + "\r\n\r\n"
           + ACTION_URL;
         SMTPMail.send (
           SMTPTest.DEFAULT_MAIL_SERVER_HOST,
           SMTPTest.DEFAULT_ADDRESS_FROM,
           mail, SUBJECT, message );
SMTPMail.send (
  SMTPTest.DEFAULT_MAIL_SERVER_HOST,
  SMTPTest.DEFAULT_ADDRESS_FROM,
  "botmaster@servername",
  SUBJECT, message );
         out.println (
           "<BLINK><B>The validation code has been mailed to \""
           + mail + "\".</B></BLINK><P>" );
         page_register ( out, user, pass, mail );
         return;
       }

       if ( !code.equals ( validation_code ( user, pass, mail ) ) )
       {
         out.println ( "<BLINK><B>Validation using code \""
           + code + "\" failed.</B></BLINK><P>" );
         page_register ( out, user, pass, mail );
         return;
       }

       try
       {
         realm.createUser ( user, pass );
       }
       catch ( InUseException  e )
       {
         if ( getUserManager ( ).getUserData ( user ) != null )
         {
           out.println ( "<BLINK><B>Username \"" + user
             + "\" already used.</B></BLINK><P>" );
           page_register ( out, "", pass, mail );
         }
         else realm.changePassword ( user, pass );
       }

       User  user1 = realm.getUser ( user );
       Group  group;
       try
       {
         group = realm.getGroup ( GROUP_NAME );
       }
       catch ( NoSuchGroupException  e )
       {
         out.println ( "<FONT COLOR=RED>ERROR<P>" );
         out.println ( "Create group \"" + GROUP_NAME
           + "\" in realm \"" + REALM_NAME + "\" first.<P>" );
         out.println ( "</FONT>" );
         throw e;
//       group = realm.makeGroup ( GROUP_NAME );
       }
       group.addMember ( user1 );
       boolean  saved = getUserManager ( ).putUserData (
         user, new UserData ( user, mail ) );
       out.println ( "<B>User \"" + user + "\" created.</B>" );
       out.println ( "<P>Update of userdata file "
         + ( saved ? "succeeded" : "failed" ) + "." );
       }
       catch ( Exception  ex )
       {
         out.println ( "<P><PRE>" );
         ex.printStackTrace ( out );
         out.println ( "</PRE><P>" );
       }
     }

     //////////////////////////////////////////////////////////////////////
     // Implementation specific instance methods
     //////////////////////////////////////////////////////////////////////

     public static void  page_reject (
       PrintStream  out ) throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       out.println ( "Automated registration is currently only available"
         + " for ANSER Employees.<P>To register as member of the general"
         + " public, please see our <A HREF=\"/agents/register.html\">"
         + "ANSER Software Agent Registration Policy</A>." );
     }

     public static void  page_register (
       PrintStream  out,
       String  user, String  pass, String  mail ) throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       out.println ( "<CENTER>" );
       out.println ( "Welcome, new user!<BR>" );
       out.println ( "Registration is accomplished in 3 steps." );
       out.println ( "<OL>" );
       out.println ( "<LI> Enter your username, password, and e-mail address." );
       out.println ( "<LI> Check your e-mail for a validation code." );
       out.println ( "<LI> Re-enter your data with your validation code." );
       out.println ( "</OL><P>" );
       out.println ( "Please enter your username and choose a password.<P>" );
       out.println ( "<FORM ACTION=\"" + ACTION_URL + "\" METHOD=POST>" );
       out.println ( "<TABLE>" );
       if ( user == null ) user = "";
       if ( pass == null ) pass = "";
       if ( mail == null ) mail = "";
       out.println ( "<TR><TD>Username Choice</TD><TD><INPUT TYPE=TEXT NAME=\"" + PARAM_USER + "\" VALUE=\"" + user + "\" SIZE=40></TD></TR>" );
       out.println ( "<TR><TD>Password Choice</TD><TD><INPUT TYPE=PASSWORD NAME=\"" + PARAM_PAS1 + "\" VALUE=\"" + pass + "\" SIZE=40></TD></TR>" );
       out.println ( "<TR><TD>Password Verify</TD><TD><INPUT TYPE=PASSWORD NAME=\"" + PARAM_PAS2 + "\" VALUE=\"" + pass + "\" SIZE=40></TD></TR>" );
       out.println ( "<TR><TD>Electronic Mail</TD><TD><INPUT TYPE=TEXT NAME=\"" + PARAM_MAIL + "\" VALUE=\"" + mail + "\" SIZE=40></TD></TR>" );
       out.println ( "<TR><TD>Validation Code</TD><TD><INPUT TYPE=TEXT NAME=\"" + PARAM_CODE + "\" SIZE=40></TD></TR>" );
       out.println ( "</TABLE>" );
       out.println ( "<INPUT TYPE=SUBMIT VALUE=\"Register\">" );
       out.println ( "</FORM>" );
       out.println ( "<P>" );
       out.println ( "Your validation code is sent to your e-mail address.<BR>" );
       out.println ( "Enter it above when you receive it.<BR>" );
       out.println ( "</CENTER>" );
     }

     public static String  validation_code (
       String  user, String  pass, String  mail )
     //////////////////////////////////////////////////////////////////////
     {
       MessageDigest  messageDigest = null;
       try
       {
         messageDigest = MessageDigest.getInstance ( "SHA-1" );
       }
       catch ( Exception  e ) { e.printStackTrace ( ); }
       String  key = user + pass + mail + VERSION;
       char [ ]  c = key.toCharArray ( );
       for ( int  i = 0; i < c.length; i++ )
       {
         messageDigest.update ( ( byte ) c [ i ] );
       }
       return StringLib.toHexString ( messageDigest.digest ( ) );
     }

     public static SharedPasswordRealm  getRealm ( )
       throws BadRealmException, NoSuchRealmException
     //////////////////////////////////////////////////////////////////////
     {
       return ( SharedPasswordRealm ) Realm.get ( REALM_NAME );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
