     //////////////////////////////////////////////////////////////////////
     //
     // Copyright (C) 1998 Analytic Services, Inc. (ANSER)
     //
     // Permission is hereby granted, free of charge, to any person
     // obtaining a copy of this software and associated documentation
     // files (the "Software"), to deal in the Software without
     // restriction, including without limitation the rights to use, copy,
     // modify, merge, publish, distribute, sublicense, and/or sell copies
     // of the Software, and to permit persons to whom the Software is
     // furnished to do so, subject to the following conditions:
     //
     // The above copyright notice and this permission notice shall be
     // included in all copies or substantial portions of the Software.
     //
     // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
     // EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
     // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
     // NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
     // HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
     // WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
     // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
     // DEALINGS IN THE SOFTWARE.
     //
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mit.app.ajax;

     import java.io.*;
     import java.util.Hashtable;

     import com.orbs.open1.mit.io.Serialize;

     /*********************************************************************
     *
     * A persistent container for UserData objects.
     *
     * @see
     *   UserData
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     * @version
     *   1998-10-29
     *********************************************************************/

     public class  UserManager
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private String     userDataFilename;
     private Hashtable  userDataHashtable;

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public  UserManager ( String  userDataFilename )
     //////////////////////////////////////////////////////////////////////
     {
       this.userDataFilename = userDataFilename;
     }

     /*********************************************************************
     * If there are any users already here by the same username,
     * they are deleted.
     *********************************************************************/
     public synchronized boolean  putUserData (
       String  username, UserData  userData )
     //////////////////////////////////////////////////////////////////////
     {
       if ( userDataHashtable == null ) load ( );
       userDataHashtable.put ( username, userData );
       return save ( );
     }

     public synchronized UserData  getUserData ( String  username )
     //////////////////////////////////////////////////////////////////////
     {
       if ( userDataHashtable == null ) load ( );
       return ( UserData ) userDataHashtable.get ( username );
     }

     private synchronized boolean  load ( )
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         userDataHashtable
           = ( Hashtable ) Serialize.load ( userDataFilename );
       }
       catch ( Exception  e ) { }
       if ( userDataHashtable == null )
       {
         userDataHashtable = new Hashtable ( );
         return false;
       }
       return true;
     }

     private synchronized boolean  save ( )
     //////////////////////////////////////////////////////////////////////
     {
       if ( userDataHashtable == null ) return false;
       try
       {
         Serialize.save ( userDataFilename, userDataHashtable );
         return true;
       }
       catch ( Exception  ex )
       {
         return false;
       }
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
