     //////////////////////////////////////////////////////////////////////
     //
     // Copyright (C) 1998 Analytic Services, Inc. (ANSER)
     //
     // Permission is hereby granted, free of charge, to any person
     // obtaining a copy of this software and associated documentation
     // files (the "Software"), to deal in the Software without
     // restriction, including without limitation the rights to use, copy,
     // modify, merge, publish, distribute, sublicense, and/or sell copies
     // of the Software, and to permit persons to whom the Software is
     // furnished to do so, subject to the following conditions:
     //
     // The above copyright notice and this permission notice shall be
     // included in all copies or substantial portions of the Software.
     //
     // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
     // EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
     // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
     // NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
     // HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
     // WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
     // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
     // DEALINGS IN THE SOFTWARE.
     //
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mit.app.calendarcourtier;

     import java.io.IOException;
     import java.util.*;

     import javax.servlet.http.*;
     import javax.servlet.ServletOutputStream;

     import com.orbs.open1.mit.app.ajax.*;
     import com.orbs.open1.mit.lang.ParseLib;
     import com.orbs.open1.mit.servlet.AgentServlet;

     /*********************************************************************
     * <P>
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft">David W. Croft</A>
     * @version
     *   1998-05-05
     *********************************************************************/

     public class  CalendarCourtierServlet extends AgentServlet
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     public static final String  TITLE   = "Calendar Courtier";
     public static final String  VERSION = "0.2.1";
     public static final String  INFO    = TITLE + " v" + VERSION
       + " Copyright 1998 ANSER";
     public static final String  MISSION_FILENAME = "CalendarCourtier.dat";

     public static final String  ACTION_URL
       = "/servlet/calendarcourtierservlet";

     public static final String  HEADER_ALIAS
       = "/java/app/calendarcourtier/header.html";
     public static final String  FOOTER_ALIAS
       = "/java/app/calendarcourtier/footer.html";

     public static final String  PARAM_INDX = "INDX";
     public static final String  PARAM_MESS = "MESS";
     public static final String  PARAM_OPTN = "OPTN";
     public static final String  PARAM_SCHE = "SCHE";
     public static final String  PARAM_YYYY = "YYYY";
     public static final String  PARAM_MM   = "MM";
     public static final String  PARAM_DD   = "DD";

     public static final String  OPTN_APND  = "APND";
     public static final String  OPTN_PREV  = "PREV";
     public static final String  OPTN_REMV  = "REMV";
     public static final String  OPTN_NEXT  = "NEXT";
     public static final String  OPTN_SAVE  = "SAVE";
     public static final String  OPTN_STAT  = "STAT";

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public String  getTitle           ( ) { return INFO; }
     public String  getVersion         ( ) { return VERSION; }
     public String  getServletInfo     ( ) { return INFO; }
     public String  getHeaderAlias     ( ) { return HEADER_ALIAS; }
     public String  getFooterAlias     ( ) { return FOOTER_ALIAS; }
     public String  getMissionFilename ( ) { return MISSION_FILENAME; }

     public void  processRequest (
       HttpServletRequest   req,
       HttpServletResponse  res,
       UserData             userData,
       Vector               mission_Vector )
       throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       ServletOutputStream  out = res.getOutputStream ( );

       String  indx = req.getParameter ( PARAM_INDX );
       String  mess = req.getParameter ( PARAM_MESS );
       String  optn = req.getParameter ( PARAM_OPTN );
       String  sche = req.getParameter ( PARAM_SCHE );
       String  yyyy = req.getParameter ( PARAM_YYYY );
       String  mm   = req.getParameter ( PARAM_MM   );
       String  dd   = req.getParameter ( PARAM_DD   );

       int  index = ParseLib.parseInt ( indx, -1 );

       if ( ( optn == null ) || optn.equals ( OPTN_STAT ) ) {
         page_stat ( out, userData, mission_Vector, index );
       } else if ( optn.equals ( OPTN_APND ) ) {
         page_apnd ( out, userData );
       } else if ( optn.equals ( OPTN_REMV ) ) {
         try {
           mission_Vector.removeElementAt ( index );
         } catch ( Exception  e ) {
           e.printStackTrace ( new java.io.PrintStream ( out ) );
         }
         page_stat ( out, userData, mission_Vector, index );
       } else if ( optn.equals ( OPTN_PREV ) ) {
         page_stat ( out, userData, mission_Vector, index - 1 );
       } else if ( optn.equals ( OPTN_NEXT ) ) {
         page_stat ( out, userData, mission_Vector, index + 1 );
       } else if ( optn.equals ( OPTN_SAVE ) ) {
         Date  start = new Date ( );
// Need to check here that they enter valid number.
// This thing bombs on leap years.  Check the Mission code as well.
         start.setYear  ( ParseLib.parseInt ( yyyy,
           start.getYear  ( ) + 1900 ) - 1900 );
         start.setMonth ( ParseLib.parseInt ( mm  ,
           start.getMonth ( ) + 1 ) - 1 );
         start.setDate  ( ParseLib.parseInt ( dd, start.getDate ( ) ) );
         CalendarCourtierMission  mission = new CalendarCourtierMission (
           userData.getName ( ), userData.getMail ( ), mess,
           ParseLib.parseInt ( sche, 0 ), start );
         mission_Vector.addElement ( mission );
         save ( );
         page_stat ( out, userData, mission_Vector, -1 );
       } else {
         page_stat ( out, userData, mission_Vector, index );
       }
     }
  
     public static void  page_apnd ( 
       ServletOutputStream  out,
       UserData             userData ) throws IOException {
     //////////////////////////////////////////////////////////////////////
       out.println ( "<CENTER>" );
       out.println ( "<FORM METHOD=POST ACTION=\"" + ACTION_URL + "\">" );
       out.println ( "<TABLE><TR><TD>" );
       out.println ( "Username</TD><TD>" );
       out.println ( userData.getName ( ) );
       out.println ( "</td></tr><tr><td>" );
       out.println ( "E-Mail Address</TD><TD>" );
       out.println ( userData.getMail ( ) );
       out.println ( "</td></tr><tr><td>" );
       out.println ( "Message</TD><TD>" );
       out.println ( "<INPUT NAME=\"" + PARAM_MESS
         + "\" TYPE=TEXT SIZE=40><P>" );
       out.println ( "</td></tr><tr><td>" );
       out.println ( "Schedule</TD><TD>" );
       out.println ( "<INPUT TYPE=RADIO NAME=\""
         + PARAM_SCHE + "\" VALUE=\""
         + CalendarCourtierMission.TYPE_ONCE + "\" CHECKED>Once" );
       out.println ( "<INPUT TYPE=RADIO NAME=\""
         + PARAM_SCHE + "\" VALUE=\""
         + CalendarCourtierMission.TYPE_YEARLY + "\">Yearly" );
       out.println ( "<INPUT TYPE=RADIO NAME=\""
         + PARAM_SCHE + "\" VALUE=\""
         + CalendarCourtierMission.TYPE_MONTHLY + "\">Monthly" );
       out.println ( "<INPUT TYPE=RADIO NAME=\""
         + PARAM_SCHE + "\" VALUE=\""
         + CalendarCourtierMission.TYPE_BIWEEKLY + "\">Biweekly" );
       out.println ( "<INPUT TYPE=RADIO NAME=\""
         + PARAM_SCHE + "\" VALUE=\""
         + CalendarCourtierMission.TYPE_WEEKLY + "\">Weekly" );
       out.println ( "<INPUT TYPE=RADIO NAME=\""
         + PARAM_SCHE + "\" VALUE=\""
         + CalendarCourtierMission.TYPE_DAILY + "\">Daily" );
       out.println ( "</td></tr><tr><td>" );
       out.println ( "Start (YYYY-MM-DD)</TD><TD>" );
       Date  now = new Date ( );
       out.println ( "<INPUT NAME=\"" + PARAM_YYYY
         + "\" TYPE=TEXT SIZE=4 VALUE=\""
         + ( now.getYear ( ) + 1900 ) + "\">-" );
       out.println ( "<INPUT NAME=\"" + PARAM_MM
         + "\" TYPE=TEXT SIZE=2 VALUE=\""
         + ( now.getMonth ( ) + 1 ) + "\">-" );
       out.println ( "<INPUT NAME=\"" + PARAM_DD
         + "\" TYPE=TEXT SIZE=2 VALUE=\"" + now.getDate  ( ) + "\">" );
       out.println ( "</td></tr><tr><td>" );
       out.println ( "<INPUT TYPE=SUBMIT VALUE=\"Save\">" );
       out.println ( "</td></tr></table>" );
       out.println ( "<INPUT TYPE=HIDDEN NAME=\"" + PARAM_OPTN
         + "\" VALUE=\"" + OPTN_SAVE + "\">" );
       out.println ( "</FORM>" );
       out.println ( "</CENTER>" );
     }

     public static void  page_stat (
       ServletOutputStream  out,
       UserData             userData,
       Vector               mission_Vector,
       int                  index ) throws IOException {
     //////////////////////////////////////////////////////////////////////
       Vector  clone = ( Vector ) mission_Vector.clone ( );
       int  size = clone.size ( );
       if ( ( index < 0 ) || ( index >= size ) ) index = size - 1;
       if ( index < 0 ) {
         page_apnd ( out, userData );
         return;
       }
       CalendarCourtierMission  mission = ( CalendarCourtierMission )
         clone.elementAt ( index );
       
       out.println ( "<CENTER>" );
       out.println ( "<table border>" );
       out.println ( "<tr><td>Record</td><td>"   + ( index + 1 ) + " of " + size + "</td></tr>" );
       out.println ( "<tr><td>Username</td><td>" + mission.getUsername ( ) + "</td></tr>" );
       out.println ( "<tr><td>E-mail</td><td>"   + mission.getUsermail ( ) + "</td></tr>" );
       out.println ( "<tr><td>Message</td><td>"  + mission.getMessage  ( ) + "</td></tr>" );
       out.println ( "<tr><td>Schedule</td><td>" + mission.getSchedule ( ) + "</td></tr>" );
       out.println ( "<tr><td>Last Run</td><td>" + mission.getLastRun  ( ) + "</td></tr>" );
       out.println ( "<tr><td>Next Run</td><td>" + mission.getNextRun  ( ) + "</td></tr>" );
       out.println ( "</table>" );
       out.println ( "<table><tr><td>" );

       out.println ( "<form action=\"" + ACTION_URL + "\" METHOD=POST>" );
       out.println ( "<input type=HIDDEN name=\"" + PARAM_INDX + "\" value=\"" + index + "\">" );
       out.println ( "<input type=HIDDEN name=\"" + PARAM_OPTN + "\" value=\"" + OPTN_PREV + "\">" );
       out.println ( "<input type=SUBMIT value=\"Previous\">" );
       out.println ( "</form></td><td>" );

       out.println ( "<form action=\"" + ACTION_URL + "\" METHOD=POST>" );
       out.println ( "<input type=HIDDEN name=\"" + PARAM_INDX + "\" value=\"" + index + "\">" );
       out.println ( "<input type=HIDDEN name=\"" + PARAM_OPTN + "\" value=\"" + OPTN_NEXT + "\">" );
       out.println ( "<input type=SUBMIT value=\"Next\">" );
       out.println ( "</form></td><td>" );

       out.println ( "<form action=\"" + ACTION_URL + "\" METHOD=POST>" );
       out.println ( "<input type=HIDDEN name=\"" + PARAM_OPTN + "\" value=\"" + OPTN_APND + "\">" );
       out.println ( "<input type=SUBMIT value=\"Add\">" );
       out.println ( "</form></td><td>" );

       out.println ( "<form action=\"" + ACTION_URL + "\" METHOD=POST>" );
       out.println ( "<input type=HIDDEN name=\"" + PARAM_INDX + "\" value=\"" + index + "\">" );
       out.println ( "<input type=HIDDEN name=\"" + PARAM_OPTN + "\" value=\"" + OPTN_REMV + "\">" );
       out.println ( "<input type=SUBMIT value=\"Remove\">" );
       out.println ( "</form></td><td>" );

       out.println ( "</tr></table>" );
       out.println ( "</CENTER>" );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
