     //////////////////////////////////////////////////////////////////////
     // Copyright (C) 1998 Analytic Services, Inc. (ANSER)
     //
     // Permission is hereby granted, free of charge, to any person
     // obtaining a copy of this software and associated documentation
     // files (the "Software"), to deal in the Software without
     // restriction, including without limitation the rights to use, copy,
     // modify, merge, publish, distribute, sublicense, and/or sell copies
     // of the Software, and to permit persons to whom the Software is
     // furnished to do so, subject to the following conditions:
     //
     // The above copyright notice and this permission notice shall be
     // included in all copies or substantial portions of the Software.
     //
     // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
     // EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
     // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
     // NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
     // HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
     // WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
     // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
     // DEALINGS IN THE SOFTWARE.
     //
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mit.io;

     import java.io.*;

     import com.orbs.open1.mit.lang.StringLib;

     /*********************************************************************
     * 
     * Static library methods to support java.io.File manipulations.
     * 
     * @see
     *   java.io.File
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     * @version
     *   1998-10-17
     *********************************************************************/

     public final class  FileLib
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private  FileLib ( ) { }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     /*********************************************************************
     * For testing purposes.
     *********************************************************************/
     public static void  main ( String [ ]  args )
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         System.out.println ( new File ( "C:" ).getCanonicalPath ( ) );

         System.out.println ( getFullPath ( new File (
           "alpha/delta/./file.txt" ) ) );
         System.out.println ( getFullPath ( new File (
           "alpha/bravo/../delta/file.txt" ) ) );
         System.out.println ( getFullPath ( new File (
           "alpha/bravo/charlie/.../delta/file.txt" ) ) );
         System.out.println ( getFullPath ( new File ( "."  ) ) );
         System.out.println ( getFullPath ( new File ( ".."  ) ) );
         System.out.println ( getFullPath ( new File ( "..\\" ) ) );
         System.out.println ( getFullPath ( new File ( "J:\\" ) ) );
         System.out.println ( getFullPath ( new File ( "C:\\" ) ) );
         System.out.println ( getFullPath ( new File ( "C:"  ) ) );

         System.out.println ( "\"" + getExtension ( "billy." ) + "\"" );
         System.out.println ( "line count:  "
           + countTextFileLines ( new File ( "FileLib.java" ), false ) );
         System.out.println ( "line count:  "
           + countTextFileLines ( new File ( "FileLib.java" ), true ) );
       }
       catch ( Exception  ex ) { ex.printStackTrace ( ); }
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public static long  countTextFileLines (
       File  file, boolean  countBlankLines )
       throws FileNotFoundException, IOException
     //////////////////////////////////////////////////////////////////////
     {
       long  lineCount = 0;
       if ( !file.isFile ( ) ) return 0;
       BufferedReader  bufferedReader
         = new BufferedReader ( new FileReader ( file ) );
       String  line;
       while ( ( line = bufferedReader.readLine ( ) ) != null )
       {
         if ( countBlankLines || ( line.trim ( ).length ( ) > 0 ) )
           lineCount++;
       }
       return lineCount;
     }

     /*********************************************************************
     * Returns the portion of the file name after the first period.
     * May return null if there is no period.
     * Will return an empty quote if the file name ends with a period.
     *********************************************************************/
     public static String  getExtension ( File  file )
     //////////////////////////////////////////////////////////////////////
     {
       return getExtension ( file.getName ( ) );
     }

     /*********************************************************************
     * Returns the portion of the file name after the first period.
     * May return null if there is no period.
     * Will return an empty quote if the file name ends with a period.
     *********************************************************************/
     public static String  getExtension ( String  fileName )
     //////////////////////////////////////////////////////////////////////
     {
       int  index = fileName.indexOf ( "." );
       if ( index < 0 ) return null;
       return fileName.substring ( index + 1 );
     }

     /*********************************************************************
     *
     * Returns File.getAbsolutePath() with the periods resolved.
     *
     * <P>
     *
     * More reliable than File.getCanonicalPath() on Wintel when some
     * drives have been substituted using the "subst" DOS command.
     *
     * <P>
     *
     * Able to resolve paths when more than two periods are in a row,
     * Wintel-style.
     *
     * <P>
     *
     * Replaces all forward and backward slashes with File.separator
     * as the first step after calling file.getAbsolutePath().
     *
     * <P>
     *
     * Examples:
     *
     * <PRE>
     *
     * alpha/delta/./file.txt                 ==> alpha/delta/file.txt
     * alpha/bravo/../delta/file.txt          ==> alpha/delta/file.txt
     * alpha/bravo/charlie/.../delta/file.txt ==> alpha/delta/file.txt
     *
     * </PRE>
     *
     *********************************************************************/
     public static String  getFullPath ( File  file )
     //////////////////////////////////////////////////////////////////////
     {
       String  path = file.getAbsolutePath ( );

       path = StringLib.replace ( path, "/" , File.separator );
       path = StringLib.replace ( path, "\\", file.separator );

       if ( path.endsWith ( File.separator + "." )
         || path.endsWith ( ".." ) )
       {
         path += File.separator;
       }

       int  index;
       while ( ( index
         = path.indexOf ( File.separator + "." + File.separator ) ) > -1 )
       {
         path = path.substring ( 0, index )
           + path.substring ( index + File.separator.length ( ) + 1 );
       }

       while ( ( index = path.indexOf ( File.separator + ".." ) ) > -1 )
       {
         int  index2 = path.indexOf ( File.separator, index + 1 );
         int  numDots = index2 - index - File.separator.length ( );
         int  index3 = index;
         for ( int  i = 0; i < numDots - 1; i++ )
         {
           index3
             = path.substring ( 0, index3 ).lastIndexOf ( File.separator );
         }
         path = path.substring ( 0, index3 ) + path.substring ( index2 );
       }

       return path;
     }

     /*********************************************************************
     * Returns the path of the file relative to the baseDir.
     *
     * Returns "" if the file and the baseDir are equal.
     *
     * <P>
     *
     * Future changes:<BR>
     * Currently, the file must be in the baseDir branch.
     * This code will change later such that "../" will be prepended
     * to the file as necessary to handle this case.
     * In the case where the file is not even on the same drive (Wintel),
     * the full path of the file may be returned or an exception might
     * be thrown.
     *********************************************************************/
     public static String  getRelativePath ( File  file, File  baseDir )
     //////////////////////////////////////////////////////////////////////
     {
       String  fileFullPath    = getFullPath ( file    );
       String  baseDirFullPath = getFullPath ( baseDir );

       if ( !baseDirFullPath.endsWith ( File.separator ) )
       {
         baseDirFullPath += File.separator;
       }

       if ( isInBranch ( fileFullPath, baseDirFullPath, File.separator ) )
       {
         if ( baseDirFullPath.length ( ) < fileFullPath.length ( ) )
         {
           return fileFullPath.substring ( baseDirFullPath.length ( ) );
         }
         else return "";
       }

       throw new IllegalArgumentException ( "file not in baseDir branch" );
     }

     /*********************************************************************
     * Returns true if the "other" path is a file or sub-directory within
     *   or at any depth below the "branch" directory path.
     * If the "branch" directory does not end with the file separator
     *   "separator", an IllegalArgumentException will be thrown.
     * Examples:<BR>
     * <PRE>
     * other = "billy"          branch = "billy/"      returns true
     * other = "billy/bob"      branch = "billy/"      returns true
     * other = "billy/bob/boy"  branch = "billy/"      returns true
     * other = "billy/"         branch = "billy/bob/"  returns false
     * other = "billy/bob"      branch = "billy/bob/"  returns true
     * other = "billy/"         branch = "billy"       throws Exception
     * </PRE>
     *********************************************************************/
     public static boolean  isInBranch (
       String  other, String  branch, String  separator )
     //////////////////////////////////////////////////////////////////////
     {
       if ( !branch.endsWith ( separator ) )
       {
         throw new IllegalArgumentException (
           "\"branch\" must end with the file separator" );
       }

       // ( branch => "billy/", other => "billy/bob" ) => true
       if ( other.startsWith ( branch ) ) return true;

       // ( branch => "billy/", other => "billy ) => true
       return ( other + separator ).startsWith ( branch );
     }

     /*********************************************************************
     * Creates the parent directories for a file specified by a combined
     *   path and filename String.
     * The name after the last separator in pathFilename is not created
     *   as a directory and is typically a filename.
     * Used when the normal File.mkdirs() operation would mistakenly create
     *   directories for the parent directories plus a directory with the
     *   same name as the filename instead of just directories for the
     *   parent directories when given the combined path and filename.
     * The pathFilename may be just a filename without a path or a
     *   a file in the root directory; in these cases, no directory will
     *   be created.
     *********************************************************************/
     public static boolean  makeParents ( String  pathFilename )
     //////////////////////////////////////////////////////////////////////
     {
       File  file = new File ( pathFilename );
       String  parent = file.getParent ( );
       if ( parent == null ) return false;
       File  dir = new File ( parent );
       return dir.mkdirs ( );
     }

     /*********************************************************************
     * Returns a String containing the contents from a file.<BR>
     * Does not do any character conversion operations.<BR>
     * Buffered.<BR>
     * Not tested.
     * @see #readTextFile
     *********************************************************************/
     public static String  readFile ( String  fileName )
       throws IOException, FileNotFoundException
     //////////////////////////////////////////////////////////////////////
     {
       BufferedInputStream  in
         = new BufferedInputStream ( new FileInputStream ( fileName ) );
       StringWriter  out = new StringWriter ( );
       int  c;
       while ( ( c = in.read ( ) ) != -1 ) out.write ( c );
       return  out.toString ( );
     }

     /*********************************************************************
     * Returns a String containing the contents from a text file.<BR>
     * Does character and end-of-line conversion operations.<BR>
     * Buffered.
     * @see #readFile
     *********************************************************************/
     public static String  readTextFile ( String  fileName )
       throws IOException, FileNotFoundException
     //////////////////////////////////////////////////////////////////////
     {
       BufferedReader in
         = new BufferedReader ( new FileReader ( fileName ) );
       StringWriter  sw = new StringWriter ( );
       PrintWriter  out = new PrintWriter ( sw, false );
       String  line;
       while ( ( line = in.readLine ( ) ) != null ) out.println ( line );
       out.flush ( );
       return sw.toString ( );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
