     //////////////////////////////////////////////////////////////////////
     //
     // Copyright (C) 1998 Analytic Services, Inc. (ANSER)
     //
     // Permission is hereby granted, free of charge, to any person
     // obtaining a copy of this software and associated documentation
     // files (the "Software"), to deal in the Software without
     // restriction, including without limitation the rights to use, copy,
     // modify, merge, publish, distribute, sublicense, and/or sell copies
     // of the Software, and to permit persons to whom the Software is
     // furnished to do so, subject to the following conditions:
     //
     // The above copyright notice and this permission notice shall be
     // included in all copies or substantial portions of the Software.
     //
     // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
     // EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
     // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
     // NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
     // HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
     // WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
     // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
     // DEALINGS IN THE SOFTWARE.
     //
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mit.net;

     import java.io.*;
     import java.net.*;
     import java.util.*;

     import com.orbs.open1.mit.lang.StringLib;

     /*********************************************************************
     *
     * Static library methods to manipulate objects related to URLs.
     *
     * @see
     *   java.net.URL
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     * @version
     *   1999-12-19
     *********************************************************************/

     public class  URLLib
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private  URLLib ( ) { }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     /*********************************************************************
     * Test method.
     *********************************************************************/
     public static void  main ( String [ ]  args )
       throws Exception
     //////////////////////////////////////////////////////////////////////
     {
       System.out.println ( decode ( "croft%40alumni.caltech.edu" ) );
       System.out.println ( parseFilename ( new URL (
         "http://nexos.anser.org:8080/~croftd" ) ) );
       System.out.println ( parseFilename ( new URL (
         "http://nexos.anser.org:8080/timc/" ) ) );
       System.out.println ( parseFilename ( new URL (
         "http://nexos.anser.org:8080/images/serverduke.gif" ) ) );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     /*********************************************************************
     * @deprecated
     *   In Java 1.2+, use URLDecoder instead.
     *********************************************************************/
     public static String  decode ( String  urlEncoded )
       throws Exception
     //////////////////////////////////////////////////////////////////////
     {
       urlEncoded = urlEncoded.replace ( '+', ' ' );
       StringBuffer  sbuf = new StringBuffer ( );
       for ( int  i = 0; i < urlEncoded.length ( ); i++ )
       {
         char  c = urlEncoded.charAt ( i );
         if ( c != '%' )
         {
           sbuf.append ( c );
         }
         else
         {
           int  code = Integer.parseInt (
             urlEncoded.substring ( i + 1, i + 3 ), 16 );
           i += 2;
           sbuf.append ( ( char ) code );
         }
       }
       return sbuf.toString ( );
     }

     /*********************************************************************
     * Guesses the MIME type based upon the filename extension in the URL
     * name.
     *********************************************************************/
     public static String  guessContentTypeFromName ( String  urlName )
     //////////////////////////////////////////////////////////////////////
     {
       String  filename = parseFilename ( urlName ).toLowerCase ( );
       if ( filename.equals   ( ""      ) ) return "text/html";
       if ( filename.endsWith ( ".htm"  ) ) return "text/html";
       if ( filename.endsWith ( ".html" ) ) return "text/html";
       if ( filename.endsWith ( ".gif"  ) ) return "image/gif";
       if ( filename.endsWith ( ".jpg"  ) ) return "image/jpg";
       if ( filename.endsWith ( ".jpeg" ) ) return "image/jpg";
       return null;
     }

     /*********************************************************************
     * Returns everything after the last '/' character in the URL name.
     *
     * @return
     *   Returns the empty String ("") if the URL name ends with a '/'.
     *   Returns null if the URL name does not have a '/' character at all.
     *********************************************************************/
     public static String  parseFilename ( URL  url )
     //////////////////////////////////////////////////////////////////////
     {
       return parseFilename ( url.toExternalForm ( ) );
     }

     /*********************************************************************
     * Returns everything after the last '/' character in the URL name.
     *
     * @return
     *   Returns the empty String ("") if the URL name ends with a '/'.
     *   Returns null if the URL name does not have a '/' character at all.
     *********************************************************************/
     public static String  parseFilename ( String  urlName )
     //////////////////////////////////////////////////////////////////////
     {
       int  index = urlName.lastIndexOf ( '/' );
       if ( index < 0 ) return null;
       return urlName.substring ( index + 1 );
     }

     /*********************************************************************
     * Saves the content of a URL to a file.
     * Returns false if "overwrite" permission is not granted and the
     *   file already exists.
     * Returns false if an error occurred.
     * Prints the stack trace if an error occurs.
     *********************************************************************/
     public static boolean  save (
       URL  url, String  fileName, boolean  overwrite )
     //////////////////////////////////////////////////////////////////////
     {
       File  file = new File ( fileName );
       if ( !overwrite && file.exists ( ) ) return false;

       BufferedInputStream   in  = null;
       BufferedOutputStream  out = null;

       try
       {
         in = new BufferedInputStream ( url.openStream ( ) );
         out = new BufferedOutputStream ( new FileOutputStream ( file ) );
         int  i;
         while ( ( i = in.read ( ) ) > -1 ) out.write ( i );
       }
       catch ( Exception  ex )
       {
         ex.printStackTrace ( );
         return false;
       }
       finally
       {
         try { in.close  ( ); } catch ( Exception  ex1 ) { }
         try { out.close ( ); } catch ( Exception  ex1 ) { }
       }

       return true;
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
