     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is
     // "com.orbs.open1.mpl.app.backpropxor.BackpropXOR",
     // created 1998-12-27.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.app.backpropxor;

     import java.applet.Applet;
     import java.awt.*;
     import java.text.DecimalFormat;

     import com.orbs.open1.mpl.gui.plot.PlotLib;
     import com.orbs.open1.mpl.math.*;

     /*********************************************************************
     * A Backpropagation neural network learning algorithm demonstration.
     *
     * @version
     *   1998-12-27
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     *********************************************************************/

     public class  BackpropXOR extends Applet implements Runnable
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     Matrix  l0_activations      = new Matrix ( 3, 1 );
     Matrix  l1_activations      = new Matrix ( 2, 1 );
     Matrix  l2_inputs           = new Matrix ( 3, 1 );
     Matrix  l2_activations      = new Matrix ( 1, 1 );
     Matrix  l1_weights          = new Matrix ( 3, 2 );
     Matrix  l2_weights          = new Matrix ( 3, 1 );
     Matrix  l1_weighted_sums    = new Matrix ( 2, 1 );
     Matrix  l2_weighted_sums    = new Matrix ( 1, 1 );
     double  output_desired      = 0.0;
     double  output_error        = 0.0;
     Matrix  l2_local_gradient   = new Matrix ( 1, 1 );
     double  learning_rate       = 0.1;
     double  momentum_constant   = 0.9;
     Matrix  l2_weights_delta    = new Matrix ( 3, 1 );
     Matrix  l2_weights_momentum = new Matrix ( 3, 1 );
     Matrix  sum_weighted_deltas = new Matrix ( 2, 1 );
     Matrix  l1_local_gradients  = new Matrix ( 2, 1 );
     Matrix  l1_weights_delta    = new Matrix ( 3, 2 );
     Matrix  l1_weights_momentum = new Matrix ( 3, 2 );

     long        total_samples = 0;

     int         iteration = 0;
     int         iterations_per_epoch = 100;
     double [ ]  squared_errors = new double [ iterations_per_epoch ];
     int         epoch = 0;
     int         epochs_max = 1000;
     double [ ]  epoch_rms_error = new double [ epochs_max ];

     double [ ] [ ]  samples = new double [ iterations_per_epoch ] [ 3 ];

     Rectangle  r;
     Rectangle  rs;

     Thread    runner;
     Image     offscreenImage;
     Graphics  offscreenGraphics;

     private boolean  please_stop = false;

     Choice     function_Choice = new Choice ( );
     TextField  learning_rate_TextField
       = new TextField ( "" + learning_rate, 4 );
     Label      learning_rate_Label = new Label ( "Learning Rate" );
     TextField  momentum_TextField
       = new TextField ( "" + momentum_constant, 4 );
     Label      momentum_Label = new Label ( "Momentum Factor" );
     Button     randomize_Button = new Button ( "Randomize Weights" );
     Button     reset_Button = new Button ( "Reset Strip" );
     Button     pause_Button = new Button ( "  Pause  " );

//     BA Z N X N X N X N A X A N B N O O
//        E O A B B A O A N N   X   X R N
//        R R         R N D O   B   A   E
//        O             D   R
//     00 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1
//     01 0 0 1 1 0 0 1 1 0 0 1 1 0 0 1 1
//     10 0 0 0 0 1 1 1 1 0 0 0 0 1 1 1 1
//     11 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1

     static final byte  ZERO =  0;
     static final byte  NOR  =  1;
     static final byte  XA   =  2;
     static final byte  NB   =  3;
     static final byte  XB   =  4;
     static final byte  NA   =  5;
     static final byte  XOR  =  6;
     static final byte  NAND =  7;
     static final byte  AND  =  8;
     static final byte  XNOR =  9;
     static final byte  A    = 10;
     static final byte  NXB  = 11;
     static final byte  B    = 12;
     static final byte  NXA  = 13;
     static final byte  OR   = 14;
     static final byte  ONE  = 15;
     int  function_selected = AND;
     String  function_String = "AND";

     static final int  font_size = 10;
     static final int  y = 60;
     static final int  xtab = font_size * 7;
     static final int  ytab = ( int ) ( ( double ) font_size * 1.5 );
     int  y2;
     int  y4;

     DecimalFormat  decimalFormat = new DecimalFormat ( "0.000000" );

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public String  getAppletInfo ( )
     //////////////////////////////////////////////////////////////////////
     {
       return "Backprop_XOR (C) Copyright 1996-1998 David Wallace Croft\n"
         + "croft@alumni.caltech.edu\n"
         + "http://www.alumni.caltech.edu/~croft/";
     }

     public boolean  action ( Event  e, Object  arg )
     //////////////////////////////////////////////////////////////////////
     {
       if ( e.target == momentum_TextField ) {
         momentum_constant = Double.valueOf (
           momentum_TextField.getText ( ) ).doubleValue ( );
         return true;
       } else if ( e.target == learning_rate_TextField ) {
         learning_rate = Double.valueOf (
           learning_rate_TextField.getText ( ) ).doubleValue ( );
         return true;
       } else if ( e.target == randomize_Button ) {
         randomize_weights ( );
         return true;
       } else if ( e.target == reset_Button ) {
         total_samples = 0;
         iteration = 0;
         epoch = 0;
         return true;
       } else if ( e.target == function_Choice ) {
         function_selected = function_Choice.getSelectedIndex ( );
         function_String = function_Choice.getSelectedItem ( );
         return true;
       } else if ( e.target == pause_Button ) {
         if ( runner != null ) {
           please_stop = true;
           pause_Button.setLabel ( "Continue" );
         } else {
           please_stop = false;
           pause_Button.setLabel ( " Pause " );
           start ( );
         }
         return true;
       } else {
         return super.action ( e, arg );
       }
     }

     public void  init ( )
     //////////////////////////////////////////////////////////////////////
     {
       offscreenImage = createImage ( size ( ).width, size ( ).height );
       offscreenGraphics = offscreenImage.getGraphics ( );
       offscreenGraphics.setFont (
         new Font ( "TimesRoman", Font.PLAIN, font_size ) );
       this.getGraphics ( ).setFont (
         new Font ( "TimesRoman", Font.PLAIN, font_size ) );
       r = new Rectangle (
         10, 10 + y, size ( ).width - 100, size ( ).height - 350 );
       y2 = 30 + y + r.height;
       y4 = y2 + ytab * 8;
       rs = new Rectangle (
         10 + r.width + 10, 10 + y,
         size ( ).height - 350, size ( ).height - 350 );
       randomize_weights ( );
       function_Choice.addItem ( "ZERO" );
       function_Choice.addItem ( "NOR" );
       function_Choice.addItem ( "XA" );
       function_Choice.addItem ( "NB" );
       function_Choice.addItem ( "XB" );
       function_Choice.addItem ( "NA" );
       function_Choice.addItem ( "XOR" );
       function_Choice.addItem ( "NAND" );
       function_Choice.addItem ( "AND" );
       function_Choice.addItem ( "XNOR" );
       function_Choice.addItem ( "A" );
       function_Choice.addItem ( "NXB" );
       function_Choice.addItem ( "B" );
       function_Choice.addItem ( "NXA" );
       function_Choice.addItem ( "OR" );
       function_Choice.addItem ( "ONE" );
       function_Choice.select ( function_selected );
       add ( function_Choice );
       add ( momentum_Label );
       add ( momentum_TextField );
       add ( learning_rate_Label );
       add ( learning_rate_TextField );
       add ( randomize_Button );
       add ( pause_Button );
       add ( reset_Button );
     }

//     public boolean mouseDown ( Event e, int x, int y ) {
     //////////////////////////////////////////////////////////////////////
//       if ( runner != null ) please_stop = true;
//       else {
//         please_stop = false;
//         start ( );
//       }
//       return true;
//     }

     public void  paint ( Graphics g )
     //////////////////////////////////////////////////////////////////////
     {
       g.setColor ( this.getBackground ( ) );
       g.fillRect ( 0, 0, size ( ).width, size ( ).height );
       g.setColor ( this.getForeground ( ) );

       plot_epochs ( r, g, epoch_rms_error );
       plot_graph  ( r, g, samples );

       g.drawString ( "Inputs", 10 + 0 * xtab, 10 + 0 * ytab + y2 );
       g.drawString ( decimalFormat.format ( l0_activations.data    [ 0 ] [ 0 ] ), 10 + 0 * xtab, 10 + 1 * ytab + y2 );
       g.drawString ( decimalFormat.format ( l0_activations.data    [ 1 ] [ 0 ] ), 10 + 0 * xtab, 10 + 2 * ytab + y2 );
       g.drawString ( decimalFormat.format ( l0_activations.data    [ 2 ] [ 0 ] ), 10 + 0 * xtab, 10 + 3 * ytab + y2 );

       g.drawString ( "Weights", 10 + 1 * xtab, 10 + 0 * ytab + y2 );
       g.drawString ( decimalFormat.format ( l1_weights.data        [ 0 ] [ 0 ] ), 10 + 1 * xtab, 10 + 1 * ytab + y2 );
       g.drawString ( decimalFormat.format ( l1_weights.data        [ 1 ] [ 0 ] ), 10 + 1 * xtab, 10 + 2 * ytab + y2 );
       g.drawString ( decimalFormat.format ( l1_weights.data        [ 2 ] [ 0 ] ), 10 + 1 * xtab, 10 + 3 * ytab + y2 );
       g.drawString ( decimalFormat.format ( l1_weights.data        [ 0 ] [ 1 ] ), 10 + 1 * xtab, 10 + 4 * ytab + y2 );
       g.drawString ( decimalFormat.format ( l1_weights.data        [ 1 ] [ 1 ] ), 10 + 1 * xtab, 10 + 5 * ytab + y2 );
       g.drawString ( decimalFormat.format ( l1_weights.data        [ 2 ] [ 1 ] ), 10 + 1 * xtab, 10 + 6 * ytab + y2 );

       g.drawString ( "Weighted Sums", 10 + 2 * xtab, 10 + 0 * ytab + y2 );
       g.drawString ( decimalFormat.format ( l1_weighted_sums.data  [ 0 ] [ 0 ] ), 10 + 2 * xtab, 10 + 1 * ytab + y2 );
       g.drawString ( decimalFormat.format ( l1_weighted_sums.data  [ 1 ] [ 0 ] ), 10 + 2 * xtab, 10 + 2 * ytab + y2 );
                                                                                     
       g.drawString ( "Hidden", 10 + 3 * xtab, 10 + 0 * ytab + y2 );
       g.drawString ( decimalFormat.format ( l2_inputs.data         [ 0 ] [ 0 ] ), 10 + 3 * xtab, 10 + 1 * ytab + y2 );
       g.drawString ( decimalFormat.format ( l2_inputs.data         [ 1 ] [ 0 ] ), 10 + 3 * xtab, 10 + 2 * ytab + y2 );
       g.drawString ( decimalFormat.format ( l2_inputs.data         [ 2 ] [ 0 ] ), 10 + 3 * xtab, 10 + 3 * ytab + y2 );

       g.drawString ( "Weights", 10 + 4 * xtab, 10 + 0 * ytab + y2 );
       g.drawString ( decimalFormat.format ( l2_weights.data        [ 0 ] [ 0 ] ), 10 + 4 * xtab, 10 + 1 * ytab + y2 );
       g.drawString ( decimalFormat.format ( l2_weights.data        [ 1 ] [ 0 ] ), 10 + 4 * xtab, 10 + 2 * ytab + y2 );
       g.drawString ( decimalFormat.format ( l2_weights.data        [ 2 ] [ 0 ] ), 10 + 4 * xtab, 10 + 3 * ytab + y2 );

       g.drawString ( "Weighted Sum", 10 + 5 * xtab, 10 + 0 * ytab + y2 );
       g.drawString ( decimalFormat.format ( l2_weighted_sums.data  [ 0 ] [ 0 ] ), 10 + 5 * xtab, 10 + 1 * ytab + y2 );

       g.drawString ( "Output", 10 + 6 * xtab, 10 + 0 * ytab + y2 );
       g.drawString ( decimalFormat.format ( l2_activations.data    [ 0 ] [ 0 ] ), 10 + 6 * xtab, 10 + 1 * ytab + y2 );

       g.drawString ( "Output Desired", 10 + 6 * xtab, 10 + 2 * ytab + y2 );
       g.drawString ( decimalFormat.format ( output_desired ), 10 + 6 * xtab, 10 + 3 * ytab + y2 );

       g.drawString ( "Output Error", 10 + 6 * xtab, 10 + 4 * ytab + y2 );
       g.drawString ( decimalFormat.format ( output_error ), 10 + 6 * xtab, 10 + 5 * ytab + y2 );

       g.drawString ( "Iterations", 10 + 7 * xtab,  10 + 0 * ytab + y2 );
       g.drawString ( "" + total_samples, 10 + 7 * xtab,  10 + 1 * ytab + y2 );

       g.drawString ( "Function", 10 + 7 * xtab,  10 + 2 * ytab + y2 );
       g.drawString ( function_String, 10 + 7 * xtab,  10 + 3 * ytab + y2 );

       g.drawString ( "L2 Gradient", 10 + 0 * xtab, 0 * ytab + y4 );
       g.drawString ( decimalFormat.format ( l2_local_gradient.data [ 0 ] [ 0 ] ), 10 + 0 * xtab, 1 * ytab + y4 );

       g.drawString ( "W2 Deltas", 10 + 1 * xtab, 0 * ytab + y4 );
       g.drawString ( decimalFormat.format ( l2_weights_delta.data  [ 0 ] [ 0 ] ), 10 + 1 * xtab, 1 * ytab + y4 );
       g.drawString ( decimalFormat.format ( l2_weights_delta.data  [ 1 ] [ 0 ] ), 10 + 1 * xtab, 2 * ytab + y4 );
       g.drawString ( decimalFormat.format ( l2_weights_delta.data  [ 2 ] [ 0 ] ), 10 + 1 * xtab, 3 * ytab + y4 );

       g.drawString ( "W2 Momentum", 10 + 2 * xtab, 0 * ytab + y4 );
       g.drawString ( decimalFormat.format ( l2_weights_momentum.data [ 0 ] [ 0 ] ), 10 + 2 * xtab, 1 * ytab + y4 );
       g.drawString ( decimalFormat.format ( l2_weights_momentum.data [ 1 ] [ 0 ] ), 10 + 2 * xtab, 2 * ytab + y4 );
       g.drawString ( decimalFormat.format ( l2_weights_momentum.data [ 2 ] [ 0 ] ), 10 + 2 * xtab, 3 * ytab + y4 );

       g.drawString ( "Sum W Deltas", 10 + 3 * xtab, 0 * ytab + y4 );
       g.drawString ( decimalFormat.format ( sum_weighted_deltas.data [ 0 ] [ 0 ] ), 10 + 3 * xtab, 1 * ytab + y4 );
       g.drawString ( decimalFormat.format ( sum_weighted_deltas.data [ 1 ] [ 0 ] ), 10 + 3 * xtab, 2 * ytab + y4 );

       g.drawString ( "L1 Gradients", 10 + 4 * xtab, 0 * ytab + y4 );
       g.drawString ( decimalFormat.format ( l1_local_gradients.data [ 0 ] [ 0 ] ), 10 + 4 * xtab, 1 * ytab + y4 );
       g.drawString ( decimalFormat.format ( l1_local_gradients.data [ 1 ] [ 0 ] ), 10 + 4 * xtab, 2 * ytab + y4 );

       g.drawString ( "W1 Deltas", 10 + 5 * xtab, 0 * ytab + y4 );
       g.drawString ( decimalFormat.format ( l1_weights_delta.data  [ 0 ] [ 0 ] ), 10 + 5 * xtab, 1 * ytab + y4 );
       g.drawString ( decimalFormat.format ( l1_weights_delta.data  [ 1 ] [ 0 ] ), 10 + 5 * xtab, 2 * ytab + y4 );
       g.drawString ( decimalFormat.format ( l1_weights_delta.data  [ 2 ] [ 0 ] ), 10 + 5 * xtab, 3 * ytab + y4 );
       g.drawString ( decimalFormat.format ( l1_weights_delta.data  [ 0 ] [ 1 ] ), 10 + 5 * xtab, 4 * ytab + y4 );
       g.drawString ( decimalFormat.format ( l1_weights_delta.data  [ 1 ] [ 1 ] ), 10 + 5 * xtab, 5 * ytab + y4 );
       g.drawString ( decimalFormat.format ( l1_weights_delta.data  [ 2 ] [ 1 ] ), 10 + 5 * xtab, 6 * ytab + y4 );

       g.drawString ( "W1 Momentum", 10 + 6 * xtab, 0 * ytab + y4 );
       g.drawString ( decimalFormat.format ( l1_weights_momentum.data [ 0 ] [ 0 ] ), 10 + 6 * xtab, 1 * ytab + y4 );
       g.drawString ( decimalFormat.format ( l1_weights_momentum.data [ 1 ] [ 0 ] ), 10 + 6 * xtab, 2 * ytab + y4 );
       g.drawString ( decimalFormat.format ( l1_weights_momentum.data [ 2 ] [ 0 ] ), 10 + 6 * xtab, 3 * ytab + y4 );
       g.drawString ( decimalFormat.format ( l1_weights_momentum.data [ 0 ] [ 1 ] ), 10 + 6 * xtab, 4 * ytab + y4 );
       g.drawString ( decimalFormat.format ( l1_weights_momentum.data [ 1 ] [ 1 ] ), 10 + 6 * xtab, 5 * ytab + y4 );
       g.drawString ( decimalFormat.format ( l1_weights_momentum.data [ 2 ] [ 1 ] ), 10 + 6 * xtab, 6 * ytab + y4 );
     }

     public void  paint_buffered ( )
     //////////////////////////////////////////////////////////////////////
     {
       paint ( offscreenGraphics );
       Graphics  g = this.getGraphics ( );
       g.drawImage ( offscreenImage, 0, 0, this );
     }

     public void  plot_epochs (
       Rectangle  r, Graphics  g, double [ ]  epochs )
     //////////////////////////////////////////////////////////////////////
     {
       g.setColor ( java.awt.Color.black );
       g.fillRect ( r.x, r.y, r.width, r.height );
       g.setColor ( java.awt.Color.white );
       g.drawRect ( r.x, r.y, r.width, r.height / 2 );
//       g.clipRect ( r.x, r.y, r.width, r.height );
       for ( int index_epoch = 1;
                 index_epoch <= epoch;
                 index_epoch++ )
       {
         PlotLib.xy ( java.awt.Color.red,
           ( double ) index_epoch, epochs [ index_epoch - 1 ],
           r, g, 1.0, ( double ) epoch, 0.0, 1.0 );
       }
//       g.clipRect ( 0, 0, size ( ).width, size ( ).height );
       g.setColor ( java.awt.Color.white );
       g.drawRect ( r.x, r.y, r.width, r.height );
       g.setColor ( this.getForeground ( ) );
     }

     public void  plot_graph (
       Rectangle  r, Graphics  g, double [ ] [ ]  samples )
     //////////////////////////////////////////////////////////////////////
     {
       Color  c;

       g.setColor ( java.awt.Color.black );
       g.fillRect ( rs.x, rs.y, rs.width, rs.height );
       for ( int index_iteration = 0;
                 index_iteration < iterations_per_epoch;
                 index_iteration++ ) {
         if ( samples [ index_iteration ] [ 2 ] >= 0.5 )
           c = java.awt.Color.green;
         else
           c = java.awt.Color.red;
         PlotLib.xy ( c,
           samples [ index_iteration ] [ 0 ],
           samples [ index_iteration ] [ 1 ],
           rs, g, 0.0, 1.0, 0.0, 1.0, 2 );
       }
       g.setColor ( java.awt.Color.white );
       g.drawRect ( rs.x, rs.y, rs.width, rs.height );
       g.setColor ( this.getForeground ( ) );
     }

     public void  randomize_weights ( )
     //////////////////////////////////////////////////////////////////////
     {
       l1_weights = l1_weights.randomizeUniform ( -1.0, 1.0 );
       l2_weights = l2_weights.randomizeUniform ( -1.0, 1.0 );
     }

     public void  run ( )
     //////////////////////////////////////////////////////////////////////
     {
       while ( !please_stop ) {

         if ( iteration == 0 ) {
           paint_buffered ( );
         }

         l2_weights = l2_weights.add ( l2_weights_delta );
//         l2_weights = l2_weights.clip ( -1.0, 1.0 );
         l1_weights = l1_weights.add ( l1_weights_delta );
//         l1_weights = l1_weights.clip ( -1.0, 1.0 );

         l0_activations = l0_activations.randomizeUniform ( 0.0, 1.0 );
         l0_activations.data [ 0 ] [ 0 ] = 1.0;

         samples [ iteration ] [ 0 ] = l0_activations.data [ 1 ] [ 0 ];
         samples [ iteration ] [ 1 ] = l0_activations.data [ 2 ] [ 0 ];

         l1_weighted_sums = Matrix.multiply (
           l1_weights.transpose ( ), l0_activations );
         l1_activations = l1_weighted_sums.sigmoid ( );
         l2_inputs.data [ 0 ] [ 0 ] = 1.0;
         l2_inputs.data [ 1 ] [ 0 ] = l1_activations.data [ 0 ] [ 0 ];
         l2_inputs.data [ 2 ] [ 0 ] = l1_activations.data [ 1 ] [ 0 ];
         l2_weighted_sums = Matrix.multiply (
           l2_weights.transpose ( ), l2_inputs );
         l2_activations = l2_weighted_sums.sigmoid ( );

         samples [ iteration ] [ 2 ] = l2_activations.data [ 0 ] [ 0 ];

         output_desired = target_function ( l0_activations );
         output_error = output_desired - l2_activations.data [ 0 ] [ 0 ];
         l2_local_gradient = l2_weighted_sums.sigmoidDerivative ( );
         l2_local_gradient = l2_local_gradient.multiply ( output_error );
// I'm not sure about the transpose below here.
         l2_weights_delta
           = Matrix.multiply ( l2_inputs, l2_local_gradient.transpose ( ) );
         l2_weights_delta = l2_weights_delta.multiply ( learning_rate );
         l2_weights_delta = l2_weights_delta.add ( l2_weights_momentum );
         l2_weights_momentum
           = l2_weights_delta.multiply ( momentum_constant );
// needs transpose or sum below?
         sum_weighted_deltas = l2_weights.submatrix ( 1, 2, 0, 0 );
         sum_weighted_deltas = Matrix.multiply (
           sum_weighted_deltas, l2_local_gradient );

         l1_local_gradients = new Matrix (
           l1_activations.rows, l1_activations.cols, 1.0 );
         l1_local_gradients = l1_local_gradients.subtract ( l1_activations );
         l1_local_gradients = Matrix.multiplyPairwise (
           l1_activations, l1_local_gradients );
         l1_local_gradients = Matrix.multiplyPairwise (
           l1_local_gradients, sum_weighted_deltas );
         l1_weights_delta = Matrix.multiply (
           l0_activations, l1_local_gradients.transpose ( ) );
         l1_weights_delta = l1_weights_delta.multiply ( learning_rate );
         l1_weights_delta = l1_weights_delta.add ( l1_weights_momentum );
         l1_weights_momentum
           = l1_weights_delta.multiply ( momentum_constant );

         squared_errors [ iteration ] = output_error * output_error;
         iteration++;
         total_samples++;
         if ( iteration % iterations_per_epoch == 0 ) {
           epoch++;
           if ( epoch >= epochs_max ) epoch = 1;
           iteration = 0;
           epoch_rms_error [ epoch - 1 ] = 0.0;
           for ( int index_iteration = 0;
                     index_iteration < iterations_per_epoch;
                     index_iteration++ ) {
             epoch_rms_error [ epoch - 1 ]
               += squared_errors [ index_iteration ];
           }
           epoch_rms_error [ epoch - 1 ] /= ( double ) iterations_per_epoch;
           epoch_rms_error [ epoch - 1 ]
             = java.lang.Math.sqrt ( epoch_rms_error [ epoch - 1 ] );
         }

//         try Thread.sleep ( 100 ); catch ( InterruptedException e );

       }
       paint_buffered ( );
       runner = null;
     }

     public void  start ( )
     //////////////////////////////////////////////////////////////////////
     {
       if ( runner == null )
       {
         runner = new Thread ( this );
         runner.start ( );
       }
     }

     public void  stop ( )
     //////////////////////////////////////////////////////////////////////
     {
       if ( runner != null )
       {
         runner.stop ( );
         runner = null;
       }
     }

     public double  target_function ( Matrix  inputs )
     //////////////////////////////////////////////////////////////////////
     {
       long  a = java.lang.Math.round ( inputs.data [ 1 ] [ 0 ] );
       long  b = java.lang.Math.round ( inputs.data [ 2 ] [ 0 ] );
       long  bit_num = 2 * b + a;
       long  mask = 1 << bit_num;
       long  masked = function_selected & mask;

       if ( masked == mask ) return 1.0; else return 0.0;
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
