     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is
     // "com.orbs.open1.mpl.app.chat.client.ChatClient",
     // created 1997-04-28.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 2000 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.app.chat.client;

     import java.applet.*;
     import java.awt.*;
     import java.io.*;
     import java.net.*;
     import java.util.*;

     import com.orbs.open1.mpl.j1d1.net.http.msg.*;
     import com.orbs.open1.mpl.j1d1.util.queue.Queue;

     import com.orbs.open1.mpl.io.*;
     import com.orbs.open1.mpl.lang.StringLib;
     import com.orbs.open1.mpl.role.Consumer;

     import com.orbs.open1.mit.util.param.ParametersLib;

     import com.orbs.open1.mpl.app.chat.*;

     /*********************************************************************
     * Chat client.
     *
     * <P>
     *
     * @version
     *   2000-04-30
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft">David W. Croft</A>
     *********************************************************************/

     public final class  ChatClient
       implements ChatConstants, Consumer, Encoder, Parser
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private final Applet              applet;

//     private final ChatFrame           chatFrame;

private final ChatPanel  chatPanel;

     private final HttpMessageClient   httpMessageClient;

     private final Queue               outgoingQueue;

     private final ChatClientInvoker   chatClientInvoker;

// final?
     private       ChatEntityID        chatEntityID;

     private       ChatThingID         chatThingID;

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public ChatClient (
       String  serverHost,
       int     serverPort,
       String  serverPath,
       Applet  applet )
     //////////////////////////////////////////////////////////////////////
     {
       this.applet = applet;

       chatEntityID = new ChatEntityID ( new Random ( ).nextLong ( ) );

       httpMessageClient = new HttpMessageClient (
         serverHost,
         serverPort,
         serverPath,
"ChatClient/1.0",
         ( Encoder ) this,
         ( Parser ) this,
"application/x-www-form-urlencoded",
         createRequestBytes ( ),
         ( Consumer ) this );

       outgoingQueue = httpMessageClient.getOutgoingQueue ( );

       chatPanel = new ChatPanel ( this, applet, outgoingQueue );

       applet.add ( chatPanel );

//       chatFrame = new ChatFrame ( this, applet, outgoingQueue );

       chatClientInvoker
         = new ChatClientInvoker ( this, chatPanel, outgoingQueue );



//       chatFrame.display ( "Selecting avatar image..." );
//       select_avatar_image ( uniqueID, DEFAULT_AVATAR_IMAGE );

//       chatFrame.display ( "Fetching list of avatar images..." );
//       send ( REQUEST + " " + LIST_AVATAR_IMAGES );

// fetching list of avatars was right here

     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public synchronized void  start ( )
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         System.out.println ( "ChatClient.start()..." );

chatPanel.viewCanvas.handleReshape ( );

         chatPanel.display ( "Requesting ID " + chatEntityID + "..." );

         outgoingQueue.append ( new ChatMessage (
           ChatMessage.CODE_REQUEST_ENTITY,
           chatEntityID,
           new String [ ] { "username", "password" } ) );

         httpMessageClient.start ( );
       }
       catch ( Exception  ex )
       {
         ex.printStackTrace ( );
       }
     }

     public synchronized void  stop ( )
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         System.out.println ( "ChatClient.stop()..." );

         httpMessageClient.stop ( );
       }
       catch ( Exception  ex )
       {
         ex.printStackTrace ( );
       }
     }

     public synchronized void  destroy ( )
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         System.out.println ( "ChatClient.destroy()..." );

         httpMessageClient.destroy ( );
       }
       catch ( Exception  ex )
       {
         ex.printStackTrace ( );
       }
     }

     //////////////////////////////////////////////////////////////////////
     // Accessor methods
     //////////////////////////////////////////////////////////////////////

     public ChatEntityID  getChatEntityID ( ) { return chatEntityID; }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public void  consume ( Object  o )
     //////////////////////////////////////////////////////////////////////
     {
       chatClientInvoker.invoke ( ( ChatMessage ) o );
     }

     public byte [ ]  encode ( Object  o )
     //////////////////////////////////////////////////////////////////////
     {
       return ParametersLib.urlEncode (
         ( ( ChatMessage ) o ).toParameters ( ) ).getBytes ( );
     }

     public Object  parse (
       InputStream  inputStream,
       int          contentLength ) throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       return ChatMessage.parse ( inputStream, contentLength );
     }

/*
     public void  setChatEntityID ( ChatEntityID  chatEntityID )
     //////////////////////////////////////////////////////////////////////
     {
       if ( chatEntityID == null )
       {
         throw new IllegalArgumentException ( "null" );
       }

       this.chatEntityID = chatEntityID;

       httpMessageClient.setRequestBytes ( createRequestBytes ( ) );
     }
*/

     public void  setChatThingID ( ChatThingID  chatThingID )
     //////////////////////////////////////////////////////////////////////
     {
       if ( chatThingID == null )
       {
         throw new IllegalArgumentException ( "null" );
       }

       this.chatThingID = chatThingID;

       chatPanel.setChatThingID ( chatThingID );
     }

/*
     public synchronized boolean  response_handle ( String  line ) {
     //////////////////////////////////////////////////////////////////////
       StringTokenizer  tokens = new StringTokenizer ( line );
       int  count = tokens.countTokens ( );
       if ( count < 2 ) return false;
       String  token = tokens.nextToken ( );
       token = tokens.nextToken ( );
       if ( token.equals ( UNIQUE_ID ) ) {
         try {
           chatPanel.setUniqueID (
             Long.parseLong ( tokens.nextToken ( ) ) );
         } catch ( Throwable  t ) { }
         return true;
       }
       if ( token.equals ( LIST_AVATAR_IMAGES ) ) {
         Vector  avatar_images_Vector = new Vector ( );
         for ( int  i = 2; i < count; i++ ) {
           avatar_images_Vector.addElement ( tokens.nextToken ( ) );
         }
         if ( chatPanel != null ) {
           chatPanel.avatar_images_update ( avatar_images_Vector );
         }
         return true;
       }
       if ( token.equals ( SELECT_AVATAR_IMAGE ) ) {
         try {
           long  id = Long.parseLong ( tokens.nextToken ( ) );
           String  avatar_image_filename = tokens.nextToken ( );
           chatPanel.select_avatar_image ( id, avatar_image_filename );
         } catch ( Throwable  t ) {
           t.printStackTrace ( );
           chatPanel.display ( t.getMessage ( ), ERROR_COLOR );
         }
         return true;
       }
       if ( token.equals ( LIST_AVATARS ) ) {
         try {
           while ( tokens.hasMoreTokens ( ) ) {
             long    avatar_id = Long.parseLong ( tokens.nextToken ( ) );
             String  avatar_image_pathfilename = tokens.nextToken ( );
             int     x = Integer.parseInt ( tokens.nextToken ( ) );
             int     y = Integer.parseInt ( tokens.nextToken ( ) );
             chatPanel.avatar_locate (
               avatar_id, avatar_image_pathfilename, x, y );
           }
         } catch ( Throwable  t ) { t.printStackTrace ( ); }
         return true;
       }
       if ( token.equals ( QUIT ) ) {
         try {
           chatPanel.quit ( Long.parseLong ( tokens.nextToken ( ) ) );
         } catch ( Throwable  t ) { t.printStackTrace ( ); }
         return true;
       }
       return false;
     }
*/

     protected synchronized void  shutdown ( Throwable  t )
     //////////////////////////////////////////////////////////////////////
     {
       t.printStackTrace ( );
       try { chatPanel.display ( t.getMessage ( ), ERROR_COLOR ); }
       catch ( Throwable  t1 ) { }
/*
       if ( shutdown ) return;
       shutdown = true;
       try { send ( REQUEST + " " + DISCONNECT ); }
       catch ( Throwable  t1 ) { }
*/
     }

     protected void  changeThingImage (
       ChatThingID  chatThingID,
       String       imageURLName )
     //////////////////////////////////////////////////////////////////////
     {
       outgoingQueue.append ( new ChatMessage (
         ChatMessage.CODE_REQUEST_IMAGE,
         chatEntityID,
         new String [ ] {
           chatThingID.toString ( ),
           imageURLName } ) );
     }

     protected void  setView ( )
     //////////////////////////////////////////////////////////////////////
     {
       if ( chatPanel != null )
       {
         Rectangle  r = chatPanel.get_view ( );

         outgoingQueue.append ( new ChatMessage (
           ChatMessage.CODE_REQUEST_VIEW,
           chatEntityID,
           new String [ ] {
             Integer.toString ( r.x      ),
             Integer.toString ( r.y      ),
             Integer.toString ( r.width  ),
             Integer.toString ( r.height ) } ) );
       }
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public void  createChatThing (
       ChatThingID  chatThingID,
       String       avatar_image_pathfilename,
       int          x,
       int          y )
     //////////////////////////////////////////////////////////////////////
     {
       chatPanel.avatarLocate (
         chatThingID, avatar_image_pathfilename, x, y );
     }

     public void  avatarRelocate (
       ChatThingID  chatThingID, int  x, int  y )
     //////////////////////////////////////////////////////////////////////
     {
       chatPanel.avatarRelocate ( chatThingID, x, y );
     }

// one goes internal, other external; confusing

     public void  avatarRelocate ( int  x, int  y )
     //////////////////////////////////////////////////////////////////////
     {
       if ( ( chatEntityID != null )
         && ( chatThingID  != null ) )
       {
         outgoingQueue.append ( new ChatMessage (
           ChatMessage.CODE_REQUEST_MOVE,
           chatEntityID,
           new String [ ] {
             chatThingID.toString ( ),
             Integer.toString ( x ),
             Integer.toString ( y ) } ) );
       }
       else
       {
         System.out.println ( "click(" + x + "," + y + ")" );
       }
     }

     public void  removeChatThing ( ChatThingID  chatThingID )
     //////////////////////////////////////////////////////////////////////
     {
       chatPanel.removeChatThing ( chatThingID );
     }

     public void  display ( String  s )
     //////////////////////////////////////////////////////////////////////
     {
       chatPanel.display ( s );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public synchronized void  reshape (
       int  x, int  y, int  width, int  height )
     //////////////////////////////////////////////////////////////////////
     {
       chatPanel.reshape ( x, y, width, height );
     }

     //////////////////////////////////////////////////////////////////////
     // Private methods
     //////////////////////////////////////////////////////////////////////

     private byte [ ]  createRequestBytes ( )
     //////////////////////////////////////////////////////////////////////
     {
       return ParametersLib.urlEncode ( new ChatMessage (
         ChatMessage.CODE_REQUEST_MESSAGE, chatEntityID, null )
         .toParameters ( ) ).getBytes ( );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
