     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is
     // "com.orbs.open1.mpl.app.chat.client.ChatReactor",
     // created 2000-01-16.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 2000 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.app.chat.client;

     import java.util.*;

     import com.orbs.open1.mpl.j1d1.util.queue.*;

     import com.orbs.open1.mpl.app.chat.*;
     import com.orbs.open1.mpl.app.chat.client.reactor.*;

     /*********************************************************************
     * Unsynchronized as single threaded invocation assumed.
     *
     * <P>
     *
     * @version
     *   2000-04-19
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft">David W. Croft</A>
     *********************************************************************/

     public class  ChatClientInvoker
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private final ChatClient  chatClient;

     private final ChatDisplay   chatDisplay;

     private final Queue       outgoingQueue;

     private final Hashtable   hashtable;

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public  ChatClientInvoker (
       ChatClient   chatClient,
       ChatDisplay  chatDisplay,
       Queue        outgoingQueue )
     //////////////////////////////////////////////////////////////////////
     {
       if ( ( chatClient    == null )
         || ( chatDisplay   == null )
         || ( outgoingQueue == null ) )
       {
         throw new IllegalArgumentException ( "null" );
       }

       this.chatClient    = chatClient;

       this.chatDisplay     = chatDisplay;

       this.outgoingQueue = outgoingQueue;

       hashtable = new Hashtable ( );

       hashtable.put ( ChatMessage.CODE_EVENT_CHAT,
         new ChatReactorChat ( chatClient ) );

       hashtable.put ( ChatMessage.CODE_EVENT_MOVE,
         new ChatReactorMove ( chatClient ) );

       hashtable.put ( ChatMessage.CODE_EVENT_REMOVE,
         new ChatReactorRemove ( chatClient ) );

       hashtable.put ( ChatMessage.CODE_EVENT_THING,
         new ChatReactorEventThing ( chatClient ) );

       hashtable.put ( ChatMessage.CODE_RESPONSE_ENTITY,
         new ChatReactorEntity ( chatClient ) );

       hashtable.put ( ChatMessage.CODE_RESPONSE_ERROR,
         new ChatReactorError ( chatClient ) );

       hashtable.put ( ChatMessage.CODE_RESPONSE_LOAD,
         new ChatReactorLoad ( chatClient ) );

       hashtable.put ( ChatMessage.CODE_RESPONSE_THING,
         new ChatReactorThing ( chatClient ) );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public void  invoke ( ChatMessage  chatMessage )
     //////////////////////////////////////////////////////////////////////
     {
       if ( chatMessage == null )
       {
         throw new IllegalArgumentException ( "null" );
       }

       String  messageCode = chatMessage.getMessageCode ( );

       ChatReactor  chatReactor
         = ( ChatReactor ) hashtable.get ( messageCode );

       if ( chatReactor != null )
       {
         ChatMessage [ ]  chatMessages = chatReactor.react ( chatMessage );

         if ( chatMessages != null )
         {
           for ( int  i = 0; i < chatMessages.length; i++ )
           {
             outgoingQueue.append ( chatMessages [ i ] );
           }
         }
       }
       else
       {
         System.err.println ( "Unknown messageCode:  " + messageCode );

         chatDisplay.display  ( "Unknown messageCode:  " + messageCode );
       }
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
