     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is
     // "com.orbs.open1.mpl.app.chat.client.ChatFrame",
     // created 1997-02-28.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.app.chat.client;

     import java.applet.*;
     import java.awt.*;
     import java.io.*;
     import java.util.*;

     import com.orbs.open1.mpl.j1d1.util.queue.Queue;

     import com.orbs.open1.mpl.gui.TextCanvas;

     import com.orbs.open1.mpl.app.chat.*;
     import com.orbs.open1.mpl.app.chat.client.applet.*;

     /*********************************************************************
     * @version
     *   2000-04-20
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft">David W. Croft</A>
     *********************************************************************/

     public final class  ChatPanel extends Panel
       implements ChatConstants, ChatDisplay
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private final ChatClient  chatClient;

     private final Applet      applet;

     private final Queue       outgoingQueue;

//   private long        uniqueID = -1;

     private boolean     shutdown = false;

     protected ViewCanvas  viewCanvas;

     protected TextCanvas  textCanvas;

     protected TextField   textField;

     private MenuItem  quit_quit = new MenuItem ( "Quit" );

     private Menu      quit_Menu = new Menu ( "Quit" );

     private CheckboxMenuItem  options_sound
       = new CheckboxMenuItem ( "Sound" );

     private Menu  options_avatar = new Menu ( "Avatar" );

     private Menu  options_Menu   = new Menu ( "Options" );

     private Vector  avatar_images_Vector;

     private Vector  avatar_MenuItem_Vector;

     private MenuItem  help_info_MenuItem = new MenuItem ( "Info" );

     private Menu      help_Menu          = new Menu ( "Help" );

     private ChatEntityID  chatEntityID;

     private ChatThingID   chatThingID;

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public ChatPanel (
       ChatClient  chatClient,
       Applet      applet,
       Queue       outgoingQueue )
     //////////////////////////////////////////////////////////////////////
     {
       this.chatClient   = chatClient;

       this.applet       = applet;

       this.outgoingQueue = outgoingQueue;

       updateTitle ( );

/*
       MenuBar  menuBar = new MenuBar ( );
       options_sound.setState ( true );
       options_Menu.add ( options_sound );
       options_avatar.disable ( );
       options_Menu.add ( options_avatar );
       menuBar.add ( options_Menu );
       quit_Menu.add ( quit_quit );
       menuBar.add ( quit_Menu );
       help_Menu.add ( help_info_MenuItem );
       menuBar.add ( help_Menu );
       setMenuBar ( menuBar );
*/

       setLayout ( null );
       add ( viewCanvas = new ViewCanvas ( chatClient, this, applet ) );
       add ( textCanvas = new TextCanvas ( Color.blue, Color.black,
         new Font ( "Helvetica", Font.BOLD, 22 ) ) );
       add ( textField  = new TextField  ( ) );

       reshape ( 0, 0, 467, 255 );
       show ( );
       textCanvas.write ( ChatApplet.APPLET_INFO, Color.green );
       viewCanvas.load_graphics ( );
     }

     protected void  setChatThingID ( ChatThingID  chatThingID )
     //////////////////////////////////////////////////////////////////////
     {
       this.chatThingID = chatThingID;

       updateTitle ( );

//       viewCanvas.setAvatarID ( chatThingID );
     }

     public synchronized void  layout ( ) {
     //////////////////////////////////////////////////////////////////////
       try {
         Rectangle  bounds = bounds ( );
         Insets  insets = insets ( );
         int  usable_height = bounds.height - insets.top - insets.bottom;
         int  textField_height = textField.preferredSize ( ).height;
         int  viewCanvas_height = ( usable_height - textField_height ) / 2;
         int  textCanvas_height = usable_height - textField_height
           - viewCanvas_height;
         viewCanvas.reshape ( insets.left, insets.top, bounds.width,
           viewCanvas_height );
         textCanvas.reshape ( insets.left, insets.top + viewCanvas_height,
           bounds.width, textCanvas_height );
         textField.reshape ( insets.left, insets.top + viewCanvas_height
           + textCanvas_height, bounds.width, textField_height );
       } catch ( Exception  e ) { e.printStackTrace ( ); }
     }

     private void  updateTitle ( )
     //////////////////////////////////////////////////////////////////////
     {
// frame       setTitle ( "ChatThingID #" + chatThingID );
     }

     public void  display ( String  line, Color  color ) {
     //////////////////////////////////////////////////////////////////////
       textCanvas.write ( line, color );
     }

     public void  display ( String  line ) {
     //////////////////////////////////////////////////////////////////////
       textCanvas.write ( line );
     }

     public boolean  handleEvent ( Event  event ) {
     //////////////////////////////////////////////////////////////////////
       try {
       if ( event.id == Event.ACTION_EVENT ) {
         if ( event.target == textField ) {
           String  line = textField.getText ( );
           if ( line != null )
           {
             outgoingQueue.append (
               new ChatMessage (
                 ChatMessage.CODE_REQUEST_CHAT,
                 chatEntityID,
                 new String [ ] { textField.getText ( ) } ) );

             textField.setText ( "" );
           }
           return true;
         }
       }

       if ( ( avatar_MenuItem_Vector != null )
         && avatar_MenuItem_Vector.contains ( event.target ) ) {
         MenuItem  avatar_MenuItem = ( MenuItem ) event.target;

         chatClient.changeThingImage (
           chatThingID, avatar_MenuItem.getLabel ( ) );

         return true;
       }

       if ( event.target == help_info_MenuItem ) {
         textCanvas.write ( ChatApplet.APPLET_INFO, Color.blue );
         return true;
       }
       if ( ( event.id == Event.WINDOW_DESTROY )
         || ( event.target == quit_quit ) ) {
         shutdown ( new RuntimeException ( "Quit" ) );
         return true;
       }
       return super.handleEvent ( event );
       } catch ( Exception  e ) {
         e.printStackTrace ( );
         return true;
       }
     }

     public void  avatar_images_update ( Vector  avatar_images_Vector ) {
     //////////////////////////////////////////////////////////////////////
       options_avatar.disable ( );
       this.avatar_images_Vector = avatar_images_Vector;
       if ( avatar_images_Vector == null ) return;
       if ( avatar_MenuItem_Vector != null ) {
         Enumeration  e = avatar_MenuItem_Vector.elements ( );
         while ( e.hasMoreElements ( ) ) {
           MenuItem  menuItem = ( MenuItem ) e.nextElement ( );
           options_avatar.remove ( menuItem );
         }
       }
       avatar_MenuItem_Vector = new Vector ( );
       Enumeration  e = avatar_images_Vector.elements ( );
       while ( e.hasMoreElements ( ) ) {
         String  avatar_image = ( String ) e.nextElement ( );
         MenuItem  menuItem = new MenuItem ( avatar_image );
         options_avatar.add ( menuItem );
         avatar_MenuItem_Vector.addElement ( menuItem );
       }
       if ( avatar_images_Vector.size ( ) > 0 ) options_avatar.enable ( );
     }

     public void  avatarRelocate (
       ChatThingID  chatThingID, int  x, int  y )
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         viewCanvas.avatarRelocate ( chatThingID, x, y );
       }
       catch ( Exception  e )
       {
         e.printStackTrace ( );
       }
     }

     protected void  avatarLocate (
       ChatThingID  chatThingID,
       String       avatar_image_pathfilename,
       int          x,
       int          y )
     //////////////////////////////////////////////////////////////////////
     {
       long  avatar_id = chatThingID.getL ( ).longValue ( );

       try
       {
         viewCanvas.avatar_locate (
           avatar_id, avatar_image_pathfilename, x, y );
       }
       catch ( Exception  e )
       {
         e.printStackTrace ( );
       }
     }

     public void  removeChatThing ( ChatThingID  chatThingID )
     //////////////////////////////////////////////////////////////////////
     {
       viewCanvas.quit ( chatThingID.getL ( ).longValue ( ) );
     }

     protected void  select_avatar_image (
       long  avatar_id, String  avatar_image_filename ) {
     //////////////////////////////////////////////////////////////////////
       viewCanvas.select_avatar_image ( avatar_id, avatar_image_filename );
     }

     protected Rectangle  get_view ( ) {
     //////////////////////////////////////////////////////////////////////
       return viewCanvas.get_view ( );
     }

     private synchronized void  shutdown ( Throwable  t ) {
     //////////////////////////////////////////////////////////////////////
       if ( shutdown ) return;
       shutdown = true;
       viewCanvas.shutdown ( );
       chatClient.shutdown ( t );
// frame       dispose ( );
     }

     protected void  quit ( long  avatar_id ) {
     //////////////////////////////////////////////////////////////////////
       viewCanvas.quit ( avatar_id );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
