     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is
     // "com.orbs.open1.mpl.app.chat.server.ChatInvoker",
     // created 1999-12-18.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.app.chat.server;

     import java.util.*;

     import com.orbs.open1.mpl.app.chat.*;
     import com.orbs.open1.mpl.app.chat.server.command.*;

     /*********************************************************************
     *
     * <P>
     *
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft">David W. Croft</A>
     * @version
     *   2000-04-25
     *********************************************************************/

     public final class  ChatInvoker
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private final ChatEntityID  chatEntityID;

     private final ChatModel     chatModel;

     private final Map           codeToCommandMap;

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public  ChatInvoker (
       ChatEntityID  chatEntityID,
       ChatModel     chatModel )
     //////////////////////////////////////////////////////////////////////
     {
       this.chatEntityID = chatEntityID;

       this.chatModel    = chatModel;

       codeToCommandMap = new HashMap ( );

       codeToCommandMap.put ( ChatMessage.CODE_REQUEST_CHAT,
         new ChatCommandChat ( chatEntityID, chatModel ) );

       codeToCommandMap.put ( ChatMessage.CODE_REQUEST_ENTITY,
         new ChatCommandEntity ( chatEntityID, chatModel ) );

       codeToCommandMap.put ( ChatMessage.CODE_REQUEST_LOAD,
         new ChatCommandLoad ( chatEntityID, chatModel ) );

       codeToCommandMap.put ( ChatMessage.CODE_REQUEST_MOVE,
         new ChatCommandMove ( chatEntityID, chatModel ) );

       codeToCommandMap.put ( ChatMessage.CODE_REQUEST_THING,
         new ChatCommandThing ( chatEntityID, chatModel ) );

       codeToCommandMap.put ( ChatMessage.CODE_REQUEST_VIEW,
         new ChatCommandView ( chatModel ) );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public ChatMessage  process ( ChatMessage  chatMessage )
     //////////////////////////////////////////////////////////////////////
     {
       String  messageCode = chatMessage.getMessageCode ( );

       ChatCommand  chatCommand
         = ( ChatCommand ) codeToCommandMap.get ( messageCode );

       if ( chatCommand != null )
       {
         return chatCommand.process ( chatMessage );
       }
       else
       {
         return new ChatMessage (
           ChatMessage.CODE_RESPONSE_ERROR,
           chatEntityID,
           new String [ ] {
             "Unknown messageCode:  " + messageCode } );
       }
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
