     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "com.orbs.open1.mpl.app.chat.ChatModel",
     // created 1999-12-18.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 2000 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.app.chat.server;

     import java.awt.Point;
     import java.awt.Rectangle;
     import java.util.*;

     import com.orbs.open1.mpl.j1d1.util.queue.*;
     import com.orbs.open1.mpl.j1d2.util.queue.*;

     import com.orbs.open1.mpl.app.chat.*;

     /*********************************************************************
     * Manages the creation, state, and destruction of things in the world
     * and their observers.
     *
     * <P>
     *
     * Unsynchronized as it is assumed that it is manipulated by a
     * single-threaded ChatSequencer.
     *
     * <P>
     *
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft">David W. Croft</A>
     * @version
     *   2000-04-28
     *********************************************************************/

     public final class  ChatModel implements ChatConstants
     //////////////////////////////////////////////////////////////////////
     // need to use memory cache
     //////////////////////////////////////////////////////////////////////
     {

     /** The ChatEntityID for the ChatModel. */

     private final ChatEntityID  modelChatEntityID;

     private final Map           chatEntityIDToOutgoingQueueMap;

     private final long          timeToLiveWithoutRenewal;

     private final Set           chatEntityIDSet;

     private final Map           chatEntityIDToLastRenewDateMap;

//   private final Map           chatEntityIDToViewRectangleMap;

     private final Map           chatEntityIDToChatThingIDSetMap;

     private final Map           chatThingIDToPositionMap;

     private final Map           chatThingIDToImageURLNameMap;

     private       long          nextChatThingIDNumber = 0;

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     /*********************************************************************
     * @param  timeToLiveWithoutRenewal
     *
     *   Milliseconds.
     *********************************************************************/
     public  ChatModel (
       ChatEntityID  modelChatEntityID,
       Map           chatEntityIDToOutgoingQueueMap,
       long          timeToLiveWithoutRenewal )
     //////////////////////////////////////////////////////////////////////
     {
       this.modelChatEntityID              = modelChatEntityID;

       this.chatEntityIDToOutgoingQueueMap = chatEntityIDToOutgoingQueueMap;

       this.timeToLiveWithoutRenewal       = timeToLiveWithoutRenewal;

       chatEntityIDSet                     = new HashSet ( );

       chatEntityIDToLastRenewDateMap      = new HashMap ( );

//     chatEntityIDToViewRectangleMap      = new HashMap ( );

       chatEntityIDToChatThingIDSetMap     = new HashMap ( );

       chatThingIDToPositionMap            = new HashMap ( );

       chatThingIDToImageURLNameMap        = new HashMap ( );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public boolean  addObserver (
       ChatEntityID  chatEntityID )
     //////////////////////////////////////////////////////////////////////
     {
System.out.println ( "ChatModel.addObserver(" + chatEntityID + ")" );

       // Do clean-up every time a new chat thing is created.

       removeExpired ( );

       boolean  result = chatEntityIDSet.add ( chatEntityID );

       if ( !result )
       {
         return false;
       }

       chatEntityIDToLastRenewDateMap.put ( chatEntityID, new Date ( ) );

       return true;
     }

     public boolean  removeObserver (
       ChatEntityID  chatEntityID )
     //////////////////////////////////////////////////////////////////////
     {
System.out.println ( "ChatModel.removeObserver(" + chatEntityID + ")" );

       boolean  result = chatEntityIDSet.remove ( chatEntityID );

       if ( !result )
       {
         return false;
       }

       chatEntityIDToLastRenewDateMap.remove  ( chatEntityID );

//     chatEntityIDToViewRectangleMap.remove  ( chatEntityID );

       Set  chatThingIDSet
         = ( Set ) chatEntityIDToChatThingIDSetMap.get ( chatEntityID );

       if ( chatThingIDSet != null )
       {
         ChatThingID [ ]  chatThingIDs = ( ChatThingID [ ] )
           chatThingIDSet.toArray ( new ChatThingID [ ] { } );

         for ( int  i = 0; i < chatThingIDs.length; i++ )
         {
           remove ( chatEntityID, chatThingIDs [ i ] );
         }
       }

       // this would be redundant as the remove() method does this
       // chatEntityIDToChatThingIDSetMap.remove ( chatEntityID );

       return true;
     }

     //////////////////////////////////////////////////////////////////////
     // Accessor methods
     //////////////////////////////////////////////////////////////////////

     public ChatThingID [ ]  getChatThingIDs ( )
     //////////////////////////////////////////////////////////////////////
     {
       return ( ChatThingID [ ] ) chatThingIDToPositionMap.keySet ( )
         .toArray ( new ChatThingID [ ] { } );
     }

     public Point  getPosition ( ChatThingID  chatThingID )
     //////////////////////////////////////////////////////////////////////
     {
       return ( Point ) chatThingIDToPositionMap.get ( chatThingID );
     }

     public String  getImageURLName ( ChatThingID  chatThingID )
     //////////////////////////////////////////////////////////////////////
     {
       return ( String ) chatThingIDToImageURLNameMap.get ( chatThingID );
     }

     //////////////////////////////////////////////////////////////////////
     // Mutator methods (generate events)
     //////////////////////////////////////////////////////////////////////

     public void  chat ( String  text )
     //////////////////////////////////////////////////////////////////////
     {
       if ( text == null )
       {
         throw new IllegalArgumentException ( "null" );
       }

       broadcast ( new ChatMessage (
         ChatMessage.CODE_EVENT_CHAT,
         modelChatEntityID,
         new String [ ] { text } ) );
     }

     public ChatThingID  createChatThingID (
       ChatEntityID  chatEntityID,
       Point         point,
       String        imageURLName )
     //////////////////////////////////////////////////////////////////////
     {
       ChatThingID  chatThingID
         = new ChatThingID ( nextChatThingIDNumber++ );

       Set  chatThingIDSet
         = ( Set ) chatEntityIDToChatThingIDSetMap.get ( chatEntityID );

       if ( chatThingIDSet == null )
       {
         chatThingIDSet = new HashSet ( );

         chatEntityIDToChatThingIDSetMap.put (
           chatEntityID, chatThingIDSet );
       }

       chatThingIDSet.add ( chatThingID );

       chatThingIDToPositionMap.put ( chatThingID, point );

       chatThingIDToImageURLNameMap.put ( chatThingID, imageURLName );

       broadcast ( new ChatMessage (
         ChatMessage.CODE_EVENT_THING,
         modelChatEntityID,
         new String [ ] {
           chatThingID.toString ( ),
           imageURLName,
           Integer.toString ( point.x ),
           Integer.toString ( point.y ) } ) );

       return chatThingID;
     }

     public void  move (
       ChatThingID   chatThingID,
       Point         point )
     //////////////////////////////////////////////////////////////////////
     {
       if ( ( chatThingID == null )
         || ( point       == null ) )
       {
         throw new IllegalArgumentException ( "null" );
       }

/*
       if ( !idToPositionMap.containsKey ( chatThingID ) )
       {
         throw new IDUnknownException ( chatThingID );
       }
*/

       Point  oldPoint
         = ( Point ) chatThingIDToPositionMap.put ( chatThingID, point );

       if ( !point.equals ( oldPoint ) )
       {
         broadcast ( new ChatMessage (
           ChatMessage.CODE_EVENT_MOVE,
           modelChatEntityID,
           new String [ ] {
             chatThingID.toString ( ),
             Integer.toString ( point.x ),
             Integer.toString ( point.y ) } ) );
       }
     }

     public boolean  remove (
       ChatEntityID  chatEntityID,
       ChatThingID   chatThingID )
     //////////////////////////////////////////////////////////////////////
     {
System.out.println ( "ChatModel.remove(" + chatThingID + ")" );

       Set  chatThingIDSet
         = ( Set ) chatEntityIDToChatThingIDSetMap.get ( chatEntityID );

       if ( chatThingIDSet == null )
       {
         return false;
       }

       if ( !chatThingIDSet.remove ( chatThingID ) )
       {
         return false;
       }

       if ( chatThingIDSet.size ( ) < 1 )
       {
         chatEntityIDToChatThingIDSetMap.remove ( chatEntityID );
       }

       chatThingIDToPositionMap.remove ( chatThingID );

       chatThingIDToImageURLNameMap.remove ( chatThingID );

       broadcast ( new ChatMessage (
         ChatMessage.CODE_EVENT_REMOVE,
         modelChatEntityID,
         new String [ ] {
           chatThingID.toString ( ) } ) );

       return true;
     }

     public void  renew ( ChatEntityID  chatEntityID )
     //////////////////////////////////////////////////////////////////////
     {
       chatEntityIDToLastRenewDateMap.put ( chatEntityID, new Date ( ) );
     }

     public void  setImageURLName (
       ChatThingID  chatThingID,
       String       imageURLName )
     //////////////////////////////////////////////////////////////////////
     {
       String  oldImageURLName = ( String )
         chatThingIDToImageURLNameMap.put ( chatThingID, imageURLName );

       if ( !imageURLName.equals ( oldImageURLName ) )
       {
         broadcast ( new ChatMessage (
           ChatMessage.CODE_EVENT_IMAGE,
           modelChatEntityID,
           new String [ ] {
             chatThingID.toString ( ),
             imageURLName } ) );
       }
     }

     public void  setView (
       ChatEntityID  chatEntityID,
       int           x,
       int           y,
       int           width,
       int           height )
     //////////////////////////////////////////////////////////////////////
     {
/*
       idToViewRectangleMap.put (
         chatEntityID, new Rectangle ( x, y, width, height ) );
*/
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     private void  broadcast ( ChatMessage  chatMessage )
     //////////////////////////////////////////////////////////////////////
     {
System.out.println ( "ChatModel.broadcast()" );

       Iterator  iterator = chatEntityIDSet.iterator ( );

       while ( iterator.hasNext ( ) )
       {
         ChatEntityID  chatEntityID = ( ChatEntityID ) iterator.next ( );

System.out.println ( "ChatModel.broadcast() to " + chatEntityID );

         Queue  outgoingQueue
           = ( Queue ) chatEntityIDToOutgoingQueueMap.get ( chatEntityID );

         if ( outgoingQueue == null )
         {
           outgoingQueue = new ListQueue ( new LinkedList ( ) );

           chatEntityIDToOutgoingQueueMap.put (
             chatEntityID, outgoingQueue );
         }

System.out.println ( "ChatModel.broadcast():  append..." );
         if ( !outgoingQueue.append ( chatMessage ) )
         {
// if queue gets full, empty and insert message to resynch whole model
         }
System.out.println ( "ChatModel.broadcast():  append done." );
       }
     }

     private void  removeExpired ( )
     //////////////////////////////////////////////////////////////////////
     {
       ChatEntityID [ ]  chatEntityIDs = ( ChatEntityID [ ] )
         chatEntityIDSet.toArray ( new ChatEntityID [ ] { } );

       long  noEarlierThanTime
         = new Date ( ).getTime ( ) - timeToLiveWithoutRenewal;

System.out.println ( "noEarlierThanTime:  " + noEarlierThanTime );

       for ( int  i = 0; i < chatEntityIDs.length; i++ )
       {
         ChatEntityID  chatEntityID = chatEntityIDs [ i ];

         Date  lastRenewDate
           = ( Date ) chatEntityIDToLastRenewDateMap.get ( chatEntityID );

         long  lastRenewTime = lastRenewDate.getTime ( );

System.out.println ( "lastRenewTime:  " + lastRenewTime );

         if ( lastRenewTime < noEarlierThanTime )
         {
           removeObserver ( chatEntityID );
         }
       }
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
