     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is
     // "com.orbs.open1.mpl.app.vaft.security.HostSecurityManager",
     // created 1999-04-26.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.app.vaft.security;

     import java.io.FileDescriptor;
     import java.net.InetAddress;

     import com.orbs.open1.mpl.security.manager.UntrustedSecurityManager;

     /*********************************************************************
     * A subclass of <I>UntrustedSecurityManager</I> that overrides
     * some restrictions.
     *
     * <P>
     *
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft">David W. Croft</A>
     * @version
     *   1998-04-26
     *********************************************************************/

     public class  HostSecurityManager extends UntrustedSecurityManager
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     public static final String [ ]  PACKAGES_FORBIDDEN
       = { "java.",
           "javax.",
           "sun.",
           "com.sun.",
           "com.orbs.pub.app.vaft."
         };

     //////////////////////////////////////////////////////////////////////
     // Check methods
     //////////////////////////////////////////////////////////////////////

// There may be a problem with classloaders for classes loaded from
// different URLs but using the same class names.

     /*********************************************************************
     * Allows the agent to use classes from the packages "java.lang",
     * "java.io", and the "vaft.*" hierarchy.  All other classes are
     * forbidden.
     *********************************************************************/
     public void  checkPackageAccess ( String  pkg )
     //////////////////////////////////////////////////////////////////////
     {
       if ( pkg.equals ( "java.lang" ) ) return;
       if ( pkg.equals ( "java.io"   ) ) return;
       if ( pkg.startsWith ( "com.orbs.pub.app.agent.vaft." ) ) return;
       super.checkPackageAccess ( pkg );
     }

     /*********************************************************************
     * Prevents any thread, trusted or untrusted, from creating new
     * classes within the following package hierarchies:
     * <UL>
     * <LI> java.*
     * <LI> javax.*
     * <LI> sun.*
     * <LI> com.sun.*
     * <LI> com.orbs.pub.app.vaft.*
     * </UL>
     *********************************************************************/
     public void  checkPackageDefinition ( String  pkg )
     //////////////////////////////////////////////////////////////////////
     {
       for ( int  i = 0; i < PACKAGES_FORBIDDEN.length; i++ )
       {
         if ( pkg.startsWith ( PACKAGES_FORBIDDEN [ i ] ) )
           throw new SecurityException ( pkg );
       }
       super.checkPackageDefinition ( pkg );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
