     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is
     // "com.orbs.open1.mpl.net.http.msg.HttpMessageClient",
     // created 2000-03-27.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 2000 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.j1d1.net.http.msg;

     import java.net.*;
     import java.util.*;

     import com.orbs.open1.mpl.j1d1.util.queue.*;

     import com.orbs.open1.mpl.io.Encoder;
     import com.orbs.open1.mpl.io.Parser;
     import com.orbs.open1.mpl.role.Consumer;

     /*********************************************************************
     * An HttpMessagePuller/Pusher facade.
     *
     * <P>
     *
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft">David W. Croft</A>
     * @version
     *   2000-04-23
     *********************************************************************/

     public final class  HttpMessageClient
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private final Consumer           consumer;

     private final HttpMessagePuller  httpMessagePuller;

     private final HttpMessagePusher  httpMessagePusher;

     private final Queue              incomingQueue;

     private final Queue              outgoingQueue;

     private final QueuePuller        incomingQueuePuller;

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public  HttpMessageClient (
       String    host,
       int       port,
       String    path,
       String    userAgent,
       Encoder   encoder,
       Parser    parser,
       String    contentType,
       byte [ ]  requestBytes,
       Consumer  consumer )
     //////////////////////////////////////////////////////////////////////
     {
       incomingQueue = new VectorQueue ( new Vector ( ) );

       outgoingQueue = new VectorQueue ( new Vector ( ) );

       this.consumer = consumer;

       if ( consumer != null )
       {
         incomingQueuePuller = new QueuePuller ( incomingQueue, consumer );
       }
       else
       {
         incomingQueuePuller = null;
       }

       httpMessagePuller = new HttpMessagePuller (
         host,
         port,
         path,
         userAgent,
         contentType, // "application/x-www-form-urlencoded"
         requestBytes,
         parser,
         incomingQueue );

       httpMessagePusher = new HttpMessagePusher (
         outgoingQueue,
         host,
         port,
         path,
         userAgent,
         encoder );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public Queue  getIncomingQueue ( ) { return incomingQueue; }

     public Queue  getOutgoingQueue ( ) { return outgoingQueue; }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public void  setRequestBytes ( byte [ ]  requestBytes )
     //////////////////////////////////////////////////////////////////////
     {
       httpMessagePuller.setRequestBytes ( requestBytes );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public synchronized void  start ( )
     //////////////////////////////////////////////////////////////////////
     {
       httpMessagePuller.start ( );

       httpMessagePusher.start ( );

       if ( incomingQueuePuller != null )
       {
         incomingQueuePuller.start ( );
       }
     }

     public synchronized void  stop ( )
     //////////////////////////////////////////////////////////////////////
     {
       httpMessagePuller.stop ( );

       httpMessagePusher.stop ( );

       if ( incomingQueuePuller != null )
       {
         incomingQueuePuller.stop ( );
       }
     }

     public synchronized void  destroy ( )
     //////////////////////////////////////////////////////////////////////
     {
       stop ( );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

// this needs to return a boolean

     public void  push ( Object  o )
     //////////////////////////////////////////////////////////////////////
     {
       outgoingQueue.append ( o );
     }

     public Object  pull ( ) throws InterruptedException
     //////////////////////////////////////////////////////////////////////
     {
       return incomingQueue.pull ( );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
