     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "com.orbs.open1.mpl.util.id.IntID"
     // created 1999-04-08.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.j1d1.util.id;

     /*********************************************************************
     * An ID implementation that uses an integer as its value.
     *
     * <P>
     *
     * Maintains a static variable "next" which it increments with each
     * new IntegerID object created using the zero-argument constructor.
     * This ensures uniqueness among IntegerID objects created within the
     * current process.
     *
     * <P>
     *
     * @see
     *   ID
     * @version
     *   2000-01-16
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     *********************************************************************/

     public class  IntID implements ID
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private static final long  serialVersionUID = 1L;

     private static int  next = 0;

     private int  i;

     //////////////////////////////////////////////////////////////////////
     // Constructor methods
     //////////////////////////////////////////////////////////////////////

     /*********************************************************************
     * this ( next++ );
     *********************************************************************/
     public  IntID ( )
     //////////////////////////////////////////////////////////////////////
     {
       this ( next++ );
     }

     public  IntID ( int  i )
     //////////////////////////////////////////////////////////////////////
     {
       this.i = i;
     }

     //////////////////////////////////////////////////////////////////////
     // Accessor method
     //////////////////////////////////////////////////////////////////////

     public int  getI ( ) { return i; }

     //////////////////////////////////////////////////////////////////////
     // ID interface methods
     //////////////////////////////////////////////////////////////////////

     public boolean  equals ( Object  other )
     //////////////////////////////////////////////////////////////////////
     {
       if ( other == null ) return false;

       if ( !getClass ( ).equals ( other.getClass ( ) ) ) return false;

       return ( i == ( ( IntID ) other ).i );
     }

     public int  hashCode ( )
     //////////////////////////////////////////////////////////////////////
     {
       return i;
     }

     public Object  clone ( )
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         return super.clone ( );
       }
       catch ( CloneNotSupportedException  ex )
       {
         // This will never happen.

         throw new RuntimeException ( );
       }
     }

     //////////////////////////////////////////////////////////////////////
     // Other methods
     //////////////////////////////////////////////////////////////////////

     /*********************************************************************
     * return Integer.toString ( i );
     *********************************************************************/
     public String  toString ( )
     //////////////////////////////////////////////////////////////////////
     {
       return Integer.toString ( i );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
