     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "com.orbs.open1.mpl.media.sound.AudioJukebox",
     // created 1999-10-07.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.media.sound;

     import java.applet.*;
     import java.net.*;
     import java.util.*;

     /*********************************************************************
     * Manages the play of multiple AudioClips.
     *
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     * @version
     *   1999-11-26
     *********************************************************************/

     public final class  AudioJukebox
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private final Map        nameToClipMap;

     private       AudioClip  audioClip;

     //////////////////////////////////////////////////////////////////////
     // Static methods
     //////////////////////////////////////////////////////////////////////

     public static void  main ( String [ ]  args )
       throws MalformedURLException
     //////////////////////////////////////////////////////////////////////
     {
       randomPlay ( 1000, args );
     }

     public static void  randomPlay (
       long        playTime,
       String [ ]  audioClipNames )
       throws MalformedURLException
     //////////////////////////////////////////////////////////////////////
     {
       Map  nameToClipMap = new HashMap ( );

       for ( int  i = 0; i < audioClipNames.length; i++ )
       {
         URL  audioClipURL = nameToClipMap.getClass ( ).getResource (
           audioClipNames [ i ] );

         AudioClip  audioClip = Applet.newAudioClip ( audioClipURL );

         nameToClipMap.put ( audioClipNames [ i ], audioClip );
       }

       AudioJukebox  audioJukebox = new AudioJukebox ( nameToClipMap );

       Random  random = new Random ( );

       while ( true )
       {
         audioJukebox.play (
           audioClipNames [ random.nextInt ( audioClipNames.length ) ] );

         try
         {
           Thread.sleep ( playTime );
         }
         catch ( InterruptedException  ex )
         {
         }
       }
     }

     //////////////////////////////////////////////////////////////////////
     // Constructor methods
     //////////////////////////////////////////////////////////////////////

     public  AudioJukebox ( Map  nameToClipMap )
     //////////////////////////////////////////////////////////////////////
     {
       if ( nameToClipMap == null )
       {
         throw new IllegalArgumentException ( "null nameToClipMap" );
       }

       this.nameToClipMap = nameToClipMap;
     }

     //////////////////////////////////////////////////////////////////////
     // Instance methods
     //////////////////////////////////////////////////////////////////////

//   addAudioClip()

//   removeAudioClip()

//   public void  play ( String  audioClipName, int  priority.....? )

     public void  play ( String  audioClipName )
     //////////////////////////////////////////////////////////////////////
     {
       stop ( );

       audioClip = ( AudioClip ) nameToClipMap.get ( audioClipName );

       try
       {
         audioClip.play ( );
       }
       catch ( NullPointerException  ex )
       {
       }
     }

     public void  stop ( )
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         audioClip.stop ( );
       }
       catch ( NullPointerException  ex )
       {
       }
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
