     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "com.orbs.open1.mpl.net.news.NewsReader",
     // created 1997-05-09.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.net.news;

     import java.net.*;
     import java.io.*;
     import java.util.*;

     /*********************************************************************
     * <P>
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft">David W. Croft</A>
     * @version
     *   1997-05-09
     *********************************************************************/

     public final class  NewsReader {
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public static final int  NNTP_PORT = 119;

     private DataInputStream  dataInputStream;
     private PrintStream      printStream;
     private Vector           newsgroups;

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public static void  main ( String [ ]  args ) throws IOException {
     //////////////////////////////////////////////////////////////////////
       String  nntp_server = args [ 0 ];
       boolean  do_list = args [ 1 ].equalsIgnoreCase ( "-l" );
       Vector  newsgroups = new Vector ( );
       if ( args [ 1 ].equalsIgnoreCase ( "-d" ) ) {
         for ( int i = 2; i < args.length; i++ ) {
           newsgroups.addElement ( args [ i ] );
         }
       }
       new NewsReader ( nntp_server, do_list, newsgroups );
     }

     public NewsReader (
       String  nntp_server, boolean  do_list, Vector  newsgroups )
       throws IOException {
     //////////////////////////////////////////////////////////////////////
       this.newsgroups = newsgroups;
       System.out.println ( "Connecting to " + nntp_server + "..." );
       Socket  socket = new Socket ( nntp_server, NNTP_PORT );
       System.out.println ( "Connected." );
       dataInputStream = new DataInputStream ( socket.getInputStream ( ) );
       printStream = new PrintStream ( socket.getOutputStream ( ) );
       try {
         String  line = dataInputStream.readLine ( );
         System.out.println ( line );
         if ( line.charAt ( 0 ) != '2' ) {
           System.out.println ( "Error: news server not found." );
           System.exit ( 1 );
         }
         if ( do_list ) listNewsgroups ( );
         displaySelectedNewsgroups ( );
       }
       finally { socket.close ( ); }
     }

     public void  listNewsgroups ( ) throws IOException {
     //////////////////////////////////////////////////////////////////////
       String  line;
       try { line = sendCommand ( "list" ); }
       catch ( Throwable  e ) {
         System.out.println ( e.getMessage ( ) );
         return;
       }
       System.out.println ( line );
       // get names of all newsgroups
       while ( true ) {
         line = dataInputStream.readLine ( );
         if ( line == null ) {
           throw new IOException ( "End of text not expected." );
         }
         if ( line.length ( ) < 1 ) continue;
         // check for end of listing
         if ( line.charAt ( 0 ) == '.' ) break;
         System.out.println ( line );
       }
     }

     public String  sendCommand ( String  cmd ) throws IOException {
     //////////////////////////////////////////////////////////////////////
       System.out.println ( "Sending command \"" + cmd + "\"..." );
       printStream.println ( cmd + "\r\n" );
       String  line = dataInputStream.readLine ( );
       System.out.println ( line );
       if ( line == null ) {
         throw new IOException ( );
       } else {
         if ( line.length ( ) < 1 ) throw new IOException ( );
         if ( line.charAt ( 0 ) != '2' ) {
           throw new IOException ( "Command \"" + cmd + "\" failed." );
         }
       }
       return line;
     }

     public String  getTextResponse ( ) throws IOException {
     //////////////////////////////////////////////////////////////////////
       String text = "";
       while ( true ) {
         String  line = dataInputStream.readLine ( );
         if ( line == null ) {
           throw new IOException ( "End of text not expected." );
         }
         if ( line.length ( ) < 1 ) continue;
         line += "\n";
         // check for end of text
         if ( line.charAt ( 0 ) == '.' ) {
           if ( line.charAt ( 1 ) != '.' ) break; // no doubt, text ended
           // we've got a doubled period here
           // collapse the doubled period to a single one
           line = line.substring ( 1, line.length ( ) );
         }
         text += line;
       }
       return text;
     }

     void displaySelectedNewsgroups ( ) throws IOException {
     //////////////////////////////////////////////////////////////////////
       Enumeration  e = newsgroups.elements ( );
       while ( e.hasMoreElements ( ) ) {
         displayNewsgroup ( ( String ) e.nextElement ( ) );
       }
     }

     void displayNewsgroup ( String  group ) throws IOException {
     //////////////////////////////////////////////////////////////////////
       String  line;
       try { line = sendCommand ( "group " + group ); }
       catch ( IOException  e ) {
         System.out.println ( e.getMessage ( ) );
         return;
       }
       System.out.println ( line );
       StringTokenizer st = new StringTokenizer ( line, " " );
       st.nextToken ( );
       st.nextToken ( );
       int  firstArticleNumber = Integer.parseInt ( st.nextToken ( ) );
       int  lastArticleNumber  = Integer.parseInt ( st.nextToken ( ) );
       System.out.println ( "First article #" + firstArticleNumber );
       System.out.println ( "Last article #"  + lastArticleNumber  );
       while ( true ) {
         try {
           // display header text
           line = sendCommand ( "head" );
           System.out.println ( line );
           System.out.println ( getTextResponse ( ) );
           // display body text
           line = sendCommand ( "body" );
           System.out.println ( line );
           System.out.println ( getTextResponse ( ) );
         }
         catch(Throwable e) {
           System.out.println ( e.getMessage ( ) );
           // there's been a problem with the current article,
           // let's try to get the next one
         }
         try { line = sendCommand ( "next" ); }
         catch ( IOException e ) {
           System.out.println ( e.getMessage ( ) );
           // an IO error, let's get out of here!
           return;
         }
       }
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
