     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "com.orbs.open1.mpl.util.SoftHashMap",
     // created 1999-04-20.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.util;

     import java.lang.ref.*;
     import java.util.*;

     /*********************************************************************
     * A Map implementation that dumps its content when memory runs low.
     *
     * <P>
     *
     * Backed by a WeakHashMap.  Note that an entry will not be garbage
     * collected if its key remains strongly reachable.
     *
     * <P>
     *
     * @see
     *   java.util.WeakHashMap
     * @see
     *   java.lang.ref.SoftReference
     *
     * @version
     *   1999-04-20
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     *********************************************************************/

     public class  SoftHashMap extends AbstractMap
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private WeakHashMap     weakHashMap    = new WeakHashMap    ( );
     private ReferenceQueue  referenceQueue = new ReferenceQueue ( );
     private Set             softSet        = new HashSet        ( );

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public static void  main ( String [ ]  args )
     //////////////////////////////////////////////////////////////////////
     {
       System.out.println ( test ( ) );
     }

     public static boolean  test ( )
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         SoftHashMap  softHashMap = new SoftHashMap ( );

         softHashMap.put ( "key", "value" );

         if ( !softHashMap.remove ( "key" ).equals ( "value" )
           || ( softHashMap.size ( ) > 0 ) )
         {
           return false;
         }

         Runtime  runtime = Runtime.getRuntime ( );

         for ( int  i = 0; i < 1000000; i++ )
         {
           if ( i % 10000 == 0 )
           {
             System.out.println (
               i + ":  "
               + softHashMap.size ( )
               + " entries, "
               + runtime.freeMemory ( )
               + " / "
               + runtime.totalMemory ( )
               + " memory usage" );
           }

           Integer  value = new Integer ( i );
           softHashMap.put ( value.toString ( ), value );
         }
       }
       catch ( Throwable  t )
       {
         t.printStackTrace ( );

         return false;
       }

       return true;
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public Set  entrySet ( )
     //////////////////////////////////////////////////////////////////////
     {
       clearQueue ( );

       return weakHashMap.entrySet ( );
     }

     public Object  put ( Object  key, Object  value )
     //////////////////////////////////////////////////////////////////////
     {
       clearQueue ( );

       softSet.add ( new SoftReference ( key, referenceQueue ) );

       return weakHashMap.put ( key, value );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public void  clearQueue ( )
     //////////////////////////////////////////////////////////////////////
     {
       Reference  reference = null;
       while ( ( reference = referenceQueue.poll ( ) ) != null )
       {
         softSet.remove ( reference );
       }
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
