     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is "com.orbs.open1.mpl.util.cache.SoftCache",
     // created 1999-04-08.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 1999 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.util.cache;

     import java.io.*;
     import java.lang.ref.*;
     import java.util.*;

     import com.orbs.open1.mpl.j1d1.util.id.ID;

     /*********************************************************************
     * A Cache implementation that dumps its content when memory is low.
     *
     * <P>
     *
     * Backed by a WeakCache.  Note that the content will not be dumped if
     * its ID remains strongly reachable.
     *
     * <P>
     *
     * @see
     *   WeakCache
     * @see
     *   java.lang.ref.SoftReference
     *
     * @version
     *   1999-04-20
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     *********************************************************************/

     public class  SoftCache implements Cache
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private WeakCache       weakCache      = new WeakCache      ( );
     private ReferenceQueue  referenceQueue = new ReferenceQueue ( );
     private Set             softSet        = new HashSet        ( );

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public ID  validate ( ID  id, ContentAccessor  contentAccessor )
       throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       clearQueue ( );

       if ( isAvailable ( id ) ) return id;

       InputStream  inputStream = contentAccessor.getInputStream ( );

       if ( inputStream == null ) return null;

       return store ( inputStream );
     }

/*
     public ID  store ( InputStream  in, ID  id ) throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       clearQueue ( );

       id = weakCache.store ( in, id );

       softSet.add ( new SoftReference ( id, referenceQueue ) );

       return id;
     }
*/

     public ID  store ( InputStream  in ) throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       clearQueue ( );

       ID  id = weakCache.store ( in );

       softSet.add ( new SoftReference ( id, referenceQueue ) );

       return id;
     }

     public InputStream  retrieve ( ID  id ) throws IOException
     //////////////////////////////////////////////////////////////////////
     {
       clearQueue ( );

       return weakCache.retrieve ( id );
     }

     public boolean  isAvailable ( ID  id )
     //////////////////////////////////////////////////////////////////////
     {
       clearQueue ( );

       return weakCache.isAvailable ( id );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public void  clearQueue ( )
     //////////////////////////////////////////////////////////////////////
     {
       Reference  reference = null;
       while ( ( reference = referenceQueue.poll ( ) ) != null )
       {
         softSet.remove ( reference );
       }
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
