     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is
     // "com.orbs.open1.mpl.util.cache.secure.SecureID",
     // created 1999-04-20.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 2000 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.util.cache.secure;

     import java.security.MessageDigest;

     import com.orbs.open1.mpl.j1d1.util.id.ID;

     /*********************************************************************
     * A concrete ID implementation that uses a secure digest to verify
     * identity.
     *
     * <P>
     *
     * @see
     *   com.orbs.open1.mpl.j1d1.util.id.ID;
     * @see
     *   java.security.MessageDigest
     *
     * @version
     *   2000-08-19
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     *********************************************************************/

     public final class  SecureID implements ID
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     private final String    algorithm;

     private final byte [ ]  digest;

     //////////////////////////////////////////////////////////////////////
     // Constructor methods
     //////////////////////////////////////////////////////////////////////

     public  SecureID (
       String    algorithm,
       byte [ ]  digest )
     //////////////////////////////////////////////////////////////////////
     {
       if ( algorithm == null )
       {
         throw new IllegalArgumentException ( "null algorithm" );
       }

       if ( digest == null )
       {
         throw new IllegalArgumentException ( "null digest" );
       }

       this.algorithm = algorithm;

       this.digest    = digest;
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public String    getAlgorithm ( ) { return algorithm; }

     public byte [ ]  getDigest    ( ) { return digest;    }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public boolean  equals ( Object  other )
     //////////////////////////////////////////////////////////////////////
     {
       if ( other == null ) return false;

       if ( !getClass ( ).equals ( other.getClass ( ) ) ) return false;

       SecureID  that = ( SecureID ) other;

       return MessageDigest.isEqual ( this.digest, that.digest )
         && this.algorithm.equals ( that.algorithm );
     }

     public int  hashCode ( )
     //////////////////////////////////////////////////////////////////////
     {
       return new String ( digest ).hashCode ( );
     }

     public Object  clone ( )
     //////////////////////////////////////////////////////////////////////
     {
       try
       {
         return super.clone ( );
       }
       catch ( CloneNotSupportedException  ex )
       {
         // this will never happen

         throw new RuntimeException ( );
       }
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
