     //////////////////////////////////////////////////////////////////////
     // The contents of this file are subject to the Mozilla Public License
     // Version 1.1 (the "License"); you may not use this file except in
     // compliance with the License.  You may obtain a copy of the License
     // at http://www.mozilla.org/MPL/
     //
     // Software distributed under the License is distributed on an "AS IS"
     // basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
     // See the License for the specific language governing rights and
     // limitations under the License.
     //
     // The Original Code is
     // "com.orbs.open1.mpl.util.state.QueuedStateMulticaster",
     // created 1999-02-07.
     //
     // The Initial Developer of the Original Code is David Wallace Croft
     // (croft@alumni.caltech.edu, http://www.alumni.caltech.edu/~croft/).
     // Portions created by David Wallace Croft are
     // Copyright (C) 2000 David Wallace Croft.  All Rights Reserved.
     //
     // Contributor(s):
     //////////////////////////////////////////////////////////////////////

     package com.orbs.open1.mpl.util.state;

     import java.util.*;

     import com.orbs.open1.mpl.j1d1.util.queue.*;
     import com.orbs.open1.mpl.j1d2.util.queue.ListQueue;

     /*********************************************************************
     *
     * Broadcasts the latest object state changes to StateListeners
     * using Queues and QueuePullers for buffering.
     * 
     * @author
     *   <A HREF="http://www.alumni.caltech.edu/~croft/">David W. Croft</A>
     * @version
     *   2000-04-30
     *********************************************************************/

     public class  QueuedStateMulticaster implements StateMulticaster
     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     {

     /** QueuePullers keyed by StateListeners. */
     private Map  queuePullerMap;

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public  QueuedStateMulticaster ( Map  queuePullerMap )
     //////////////////////////////////////////////////////////////////////
     {
       this.queuePullerMap = queuePullerMap;
     }

     /*********************************************************************
     * this ( new HashMap ( ) );
     *********************************************************************/
     public  QueuedStateMulticaster ( )
     //////////////////////////////////////////////////////////////////////
     {
       this ( new HashMap ( ) );
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////

     public void  update ( State  state )
     //////////////////////////////////////////////////////////////////////
     {
       QueuePuller [ ]  queuePullers = null;

       synchronized ( queuePullerMap )
       {
         queuePullers = ( QueuePuller [ ] )
           queuePullerMap.values ( ).toArray ( new QueuePuller [ 0 ] );
       }

       for ( int  i = 0; i < queuePullers.length; i++ )
       {
         queuePullers [ i ].replace ( state );
       }
     }

     public synchronized boolean  addStateListener (
       StateListener  stateListener )
     //////////////////////////////////////////////////////////////////////
     {
       QueuePuller  queuePuller = new QueuePuller (
         new ListQueue ( new LinkedList ( ) ),
         new StateListenerConsumer ( stateListener ) );

       if ( !queuePullerMap.containsKey ( stateListener ) )
       {
         queuePullerMap.put ( stateListener, queuePuller );
         queuePuller.start ( );
         return true;
       }
       else
       {
         return false;
       }
     }

     public synchronized boolean  removeStateListener (
       StateListener  stateListener )
     //////////////////////////////////////////////////////////////////////
     {
       QueuePuller  queuePuller
         = ( QueuePuller ) queuePullerMap.remove ( stateListener );
       if ( queuePuller == null ) return false;
       queuePuller.stop ( );
       return true;
     }

     //////////////////////////////////////////////////////////////////////
     //////////////////////////////////////////////////////////////////////
     }
